@@ -1,4 +1,4 @@
-/* -*- mode: C -*- Time-stamp: "2011-11-02 20:37:12 jemarch"
+/* -*- mode: C -*- Time-stamp: "2012-02-25 15:55:25 jemarch"
  *
  *       File:         rec-crypt.c
  *       Date:         Fri Aug 26 19:50:51 2011
@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2011 Jose E. Marchesi */
+/* Copyright (C) 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,10 +37,12 @@
 #define AESV2_BLKSIZE 16
 #define AESV2_KEYSIZE 16
 
+#define SALT_SIZE 4
+
 bool
 rec_encrypt (char   *in,
              size_t  in_size,
-             char   *password,
+             const char   *password,
              char  **out,
              size_t *out_size)
 {
@@ -115,26 +117,37 @@ rec_encrypt (char   *in,
   if (gcry_cipher_setkey (handler, key, AESV2_KEYSIZE)
       != GPG_ERR_NO_ERROR)
     {
+      gcry_cipher_close (handler);
       return false;
     }
 
-  for (i = 0; i < AESV2_BLKSIZE; i++)
+  gcry_create_nonce (iv, SALT_SIZE);
+  for (i = SALT_SIZE; i < AESV2_BLKSIZE; i++)
     {
       iv[i] = i;
     }
-  gcry_cipher_setiv (handler, iv, AESV2_BLKSIZE);
+  if (gcry_cipher_setiv (handler, iv, AESV2_BLKSIZE)
+      != GPG_ERR_NO_ERROR)
+    {
+      gcry_cipher_close (handler);
+      return false;
+    }
 
-  /* Encrypt the data.  */
-  *out_size = real_in_size;
+  *out_size = real_in_size + SALT_SIZE;
   *out = malloc (*out_size);
 
+  /* Append salt at the end of the output.  */
+  memcpy (*out + real_in_size, iv, SALT_SIZE);
+
+  /* Encrypt the data.  */
   if (gcry_cipher_encrypt (handler,
                            *out,
-                           *out_size,
+                           real_in_size,
                            real_in,
                            real_in_size) != 0)
     {
       /* Error.  */
+      gcry_cipher_close (handler);
       return false;
     }
 
@@ -147,7 +160,7 @@ rec_encrypt (char   *in,
 bool
 rec_decrypt (char   *in,
              size_t  in_size,
-             char   *password,
+             const char   *password,
              char  **out,
              size_t *out_size)
 {
@@ -156,8 +169,13 @@ rec_decrypt (char   *in,
   size_t password_size;
   char key[AESV2_KEYSIZE];
   char iv[AESV2_BLKSIZE];
-  
-  if ((in_size % AESV2_BLKSIZE) != 0)
+  size_t salt_size = 0;
+
+  if (((in_size - SALT_SIZE) % AESV2_BLKSIZE) == 0)
+    {
+      salt_size = SALT_SIZE;
+    }
+  else if ((in_size % AESV2_BLKSIZE) != 0)
     {
       return false;
     }
@@ -183,25 +201,34 @@ rec_decrypt (char   *in,
       != GPG_ERR_NO_ERROR)
     {
       printf ("error setting key\n");
+      gcry_cipher_close (handler);
       return false;
     }
 
-  for (i = 0; i < AESV2_BLKSIZE; i++)
+  /* Extract salt at the end of the output.  */
+  memcpy (iv, in + in_size - salt_size, salt_size);
+  for (i = salt_size; i < AESV2_BLKSIZE; i++)
     {
       iv[i] = i;
     }
-  gcry_cipher_setiv (handler, iv, AESV2_BLKSIZE);
+  if (gcry_cipher_setiv (handler, iv, AESV2_BLKSIZE)
+      != GPG_ERR_NO_ERROR)
+    {
+      gcry_cipher_close (handler);
+      return false;
+    }
 
   /* Decrypt the data.  */
-  *out_size = in_size;
+  *out_size = in_size - salt_size;
   *out = malloc (*out_size);
   if (gcry_cipher_decrypt (handler,
                            *out,
                            *out_size,
                            in,
-                           in_size) != 0)
+                           in_size - salt_size) != 0)
     {
       /* Error.  */
+      gcry_cipher_close (handler);
       return false;
     }
 
@@ -219,6 +246,7 @@ rec_decrypt (char   *in,
 
       if (crc32 (*out, strlen(*out) - 4) != crc)
         {
+          gcry_cipher_close (handler);
           return false;
         }
 
@@ -226,6 +254,7 @@ rec_decrypt (char   *in,
     }
   else
     {
+      gcry_cipher_close (handler);
       return false;
     }
 
@@ -238,11 +267,11 @@ rec_decrypt (char   *in,
 bool
 rec_encrypt_record (rec_rset_t rset,
                     rec_record_t record,
-                    char *password)
+                    const char *password)
 {
   rec_field_t field;
   bool res;
-  rec_field_name_t field_name;
+  const char *field_name;
   rec_fex_t confidential_fields;
   size_t i, k, num_fields;
 
@@ -276,7 +305,7 @@ rec_encrypt_record (rec_rset_t rset,
 
 bool
 rec_encrypt_field (rec_field_t field,
-                   char *password)
+                   const char *password)
 {
   char *field_value;
   char *field_value_encrypted;
@@ -285,6 +314,10 @@ rec_encrypt_field (rec_field_t field,
   char *aux;
 
   field_value = strdup (rec_field_value (field));
+  if (!field_value)
+    {
+      return false;
+    }
 
   /* Make sure the field is not already encrypted.  */
   if ((strlen (rec_field_value (field)) >= strlen (REC_ENCRYPTED_PREFIX))
@@ -340,9 +373,9 @@ rec_encrypt_field (rec_field_t field,
 
 bool
 rec_decrypt_field (rec_field_t field,
-                   char *password)
+                   const char *password)
 {
-  char *field_value;
+  const char *field_value;
   char *base64_decoded;
   size_t base64_decoded_size;
   char *decrypted_value;
@@ -392,12 +425,12 @@ rec_decrypt_field (rec_field_t field,
 bool
 rec_decrypt_record (rec_rset_t rset,
                     rec_record_t record,
-                    char *password)
+                    const char *password)
 {
   bool res = true;
   size_t i, num_fields, k;
   rec_field_t field;
-  rec_field_name_t field_name;
+  const char *field_name;
   rec_fex_t confidential_fields;
 
   if (rset)
