@@ -58,6 +58,8 @@ static int rec_int_check_record_unique (rec_rset_t rset, rec_record_t record,
                                         rec_buf_t errors);
 static int rec_int_check_record_prohibit (rec_rset_t rset, rec_record_t record,
                                           rec_buf_t errors);
+static int rec_int_check_record_sex_constraints (rec_rset_t rset, rec_record_t record,
+                                                 rec_buf_t errors);
 
 #if defined REC_CRYPT_SUPPORT
 static int rec_int_check_record_secrets (rec_rset_t rset, rec_record_t record,
@@ -65,7 +67,7 @@ static int rec_int_check_record_secrets (rec_rset_t rset, rec_record_t record,
 #endif
 
 static int rec_int_merge_remote (rec_rset_t rset, rec_buf_t errors);
-static bool rec_int_rec_type_p (char *str);
+static bool rec_int_rec_type_p (const char *str);
 
 /* The following macros are used by some functions in this file to
    reduce verbosity.  */
@@ -169,7 +171,7 @@ rec_int_check_rset (rec_db_t db,
       if (num_records != min_records)
         {
           ADD_ERROR (errors,
-                     _("%s: error: the number of records of type %s shall be %zd.\n"),
+                     _("%s: error: the number of records of type %s should be %zd.\n"),
                      rec_rset_source (rset), rec_rset_type (rset), min_records);
           res++;
         }
@@ -229,8 +231,8 @@ rec_int_check_record (rec_db_t db,
 #if defined REC_CRYPT_SUPPORT
     + rec_int_check_record_secrets   (rset, record, errors)
 #endif
-    + rec_int_check_record_prohibit  (rset, record, errors);
-
+    + rec_int_check_record_prohibit  (rset, record, errors)
+    + rec_int_check_record_sex_constraints (rset, record, errors);
 
   return res;
 }
@@ -241,95 +243,76 @@ rec_int_check_field_type (rec_db_t db,
                           rec_field_t field,
                           rec_buf_t errors)
 {
-  bool res;
-  rec_field_name_t field_name;
-  const char *rset_name;
-  rec_rset_t referred_rset;
+  bool res = true;
   rec_type_t type;
-  rec_type_t referring_type;
-  rec_type_t referred_type;
   char *errors_str;
 
   res = true;
-  rset_name = NULL;
-  referred_rset = NULL;
-  referred_type = NULL;
-  referring_type = NULL;
 
-  field_name = rec_field_name (field);
 
-  /* Get the proper type to check 'field' with.  The algorithm differs
-     depending on the kind of field:
-     
-     - For normal fields, we check with the type from the type
-       registry of 'rset', if any.
+  /* Get the proper type to check 'field' with, checking with the type
+     from the type registry of 'rset', if any.  */
 
-     - For compound fields (reference), we check with the type from
-       the type registry of the referenced rset, if such an rset is
-       found.
+  type = rec_rset_get_field_type (rset, rec_field_name (field));
 
-     Note that if a type declaration in the referring rset exist for
-     the field and a conflict arises then the type descriptor in the
-     referred record takes precedence and a warning is emitted.  (XXX:
-     maybe a configurable error?).
-
-  */
-
-  /* Get the referred type, if any.  */
-  if (rec_field_name_size (field_name) > 1)
-    {
-      rset_name = rec_field_name_get (field_name, 0);
-      if (rset_name)
-        {
-          referred_rset = rec_db_get_rset_by_type (db, rset_name);
-        }
-
-      if (referred_rset)
-        {
-          referred_type = rec_rset_get_field_type (referred_rset,
-                                                   rec_field_name (field));
-        }
-    }
-
-  /* Get the referring type, if any.  */
-  referring_type = rec_rset_get_field_type (rset, rec_field_name (field));
-
-  /* The referring type takes precedence.  */
-  if (referring_type)
-    {
-      if (referred_type
-          && (!rec_type_equal_p (referred_type, referring_type))
-          && errors)
-        {
-          ADD_ERROR (errors,
-                     _("%s:%s: warning: type %s collides with referred type %s in the rset %s.\n"),
-                     rec_field_source (field), rec_field_location_str (field),
-                     rec_type_kind_str (referred_type), rec_type_kind_str (referring_type),
-                     rset_name);
-        }
-
-      type = referring_type;
-    }
-  else
-    {
-      type = referred_type;
-    }
-
-  /* Check the field with the type.  */
+  /* Check the field with the type.  This is done by simply invoking
+     rec_type_check on the field value.  An exception to this is the
+     'rec' type.  The 'rec' type is used to implement foreign keys,
+     and its effect on the type integrity system is that the value of
+     the field must be considered to be of whatever type the primary
+     key of the referred record set is.  */
 
   if (type)
     {
-      if (!rec_type_check (type, rec_field_value (field), &errors_str))
+      if (rec_type_kind (type) == REC_TYPE_REC)
         {
-          if (errors)
+          /* Get the name of the referred record set.  Check the type
+             if and only if:
+             
+             - The referred rset exists in DB and
+             - The referred rset has a primary key.
+             - The primary key of the referred rset has a type.
+          */
+
+          const char *rset_type = rec_type_rec (type);
+          rec_rset_t rset = rec_db_get_rset_by_type (db, rset_type);
+
+          if (rset)
             {
-              ADD_ERROR (errors,
-                         "%s:%s: error: %s\n",
-                         rec_field_source (field), rec_field_location_str (field),
-                         errors_str);
+              const char *key = rec_rset_key (rset);
+              rec_type_t key_type = rec_rset_get_field_type (rset, key);
+
+              if (key_type)
+                {
+                  if (!rec_type_check (key_type, rec_field_value (field), &errors_str))
+                    {
+                      if (errors)
+                        {
+                          ADD_ERROR (errors,
+                                     "%s:%s: error: %s\n",
+                                     rec_field_source (field), rec_field_location_str (field),
+                                     errors_str);
+                        }
+                      free (errors_str);
+                      res = false;
+                    }
+                }
+            }
+        }
+      else
+        {
+          if (!rec_type_check (type, rec_field_value (field), &errors_str))
+            {
+              if (errors)
+                {
+                  ADD_ERROR (errors,
+                             "%s:%s: error: %s\n",
+                             rec_field_source (field), rec_field_location_str (field),
+                             errors_str);
+                }
+              free (errors_str);
+              res = false;
             }
-          free (errors_str);
-          res = false;
         }
     }
 
@@ -375,8 +358,7 @@ rec_int_check_record_mandatory (rec_rset_t rset,
   int res;
   rec_record_t descriptor;
   rec_fex_t mandatory_fex;
-  rec_field_name_t mandatory_field_name;
-  char *mandatory_field_str;
+  const char *mandatory_field_name;
   rec_field_t field;
   size_t i, j, num_fields;
   
@@ -401,7 +383,6 @@ rec_int_check_record_mandatory (rec_rset_t rset,
           for (j = 0; j < rec_fex_size (mandatory_fex); j++)
             {
               mandatory_field_name = rec_fex_elem_field_name (rec_fex_get (mandatory_fex, j));
-              mandatory_field_str = rec_fex_elem_field_name_str (rec_fex_get (mandatory_fex, j));
 
               if (rec_record_get_num_fields_by_name (record, mandatory_field_name)
                   == 0)
@@ -410,7 +391,7 @@ rec_int_check_record_mandatory (rec_rset_t rset,
                              _("%s:%s: error: mandatory field '%s' not found in record\n"),
                              rec_record_source (record),
                              rec_record_location_str (record),
-                             mandatory_field_str);
+                             mandatory_field_name);
                   res++;
                 }
             }
@@ -428,8 +409,7 @@ rec_int_check_record_unique (rec_rset_t rset,
   int res;
   rec_record_t descriptor;
   rec_fex_t unique_fex;
-  rec_field_name_t unique_field_name;
-  char *unique_field_str;
+  const char *unique_field_name;
   rec_field_t field;
   size_t i, j, num_fields;
   
@@ -454,7 +434,6 @@ rec_int_check_record_unique (rec_rset_t rset,
           for (j = 0; j < rec_fex_size (unique_fex); j++)
             {
               unique_field_name = rec_fex_elem_field_name (rec_fex_get (unique_fex, j));
-              unique_field_str = rec_fex_elem_field_name_str (rec_fex_get (unique_fex, j));
 
               if (rec_record_get_num_fields_by_name (record, unique_field_name)
                   > 1)
@@ -463,7 +442,7 @@ rec_int_check_record_unique (rec_rset_t rset,
                              _("%s:%s: error: field '%s' should be unique in this record\n"),
                              rec_record_source (record),
                              rec_record_location_str (record),
-                             unique_field_str);
+                             unique_field_name);
                   res++;
                 }
             }
@@ -481,8 +460,7 @@ rec_int_check_record_prohibit (rec_rset_t rset,
   int res;
   rec_record_t descriptor;
   rec_fex_t prohibit_fex;
-  rec_field_name_t prohibit_field_name;
-  char *prohibit_field_str;
+  const char *prohibit_field_name;
   rec_field_t field;
   size_t i, j, num_fields;
   
@@ -507,7 +485,6 @@ rec_int_check_record_prohibit (rec_rset_t rset,
           for (j = 0; j < rec_fex_size (prohibit_fex); j++)
             {
               prohibit_field_name = rec_fex_elem_field_name (rec_fex_get (prohibit_fex, j));
-              prohibit_field_str = rec_fex_elem_field_name_str (rec_fex_get (prohibit_fex, j));
 
               if (rec_record_get_num_fields_by_name (record, prohibit_field_name)
                   > 0)
@@ -516,7 +493,7 @@ rec_int_check_record_prohibit (rec_rset_t rset,
                              _("%s:%s: error: prohibited field '%s' found in record\n"),
                              rec_record_source (record),
                              rec_record_location_str (record),
-                             prohibit_field_str);
+                             prohibit_field_name);
                   res++;
                 }
             }
@@ -526,6 +503,34 @@ rec_int_check_record_prohibit (rec_rset_t rset,
   return res;
 }
 
+static int
+rec_int_check_record_sex_constraints (rec_rset_t rset,
+                                      rec_record_t record,
+                                      rec_buf_t errors)
+{
+  int res = 0;
+  size_t i = 0;
+  size_t num_constraints = rec_rset_num_sex_constraints (rset);
+
+  for (i = 0; i < num_constraints; i++)
+    {
+      bool status = false;
+      rec_sex_t sex = rec_rset_sex_constraint (rset, i);
+
+      if (!rec_sex_eval (sex, record, &status))
+        {
+          ADD_ERROR (errors,
+                     _("%s:%s: error: %%constraint[%d] violated in record\n"),
+                     rec_record_source (record),
+                     rec_record_location_str (record),
+                     i);
+          res++;
+        }
+    }
+
+  return res;
+}
+
 #if defined REC_CRYPT_SUPPORT
 
 static int
@@ -576,7 +581,7 @@ rec_int_check_record_key (rec_rset_t rset,
   rec_record_t descriptor;
   rec_record_t other_record;
   rec_mset_iterator_t iter;
-  rec_field_name_t key_field_name;
+  char *key_field_name;
   rec_field_t field;
   rec_field_t key;
   rec_field_t other_key;
@@ -659,13 +664,13 @@ rec_int_check_record_key (rec_rset_t rset,
                                  _("%s:%s: error: duplicated key value in field '%s' in record\n"),
                                  rec_record_source (orig_record),
                                  rec_record_location_str (orig_record),
-                                 rec_field_name_str (key));
+                                 rec_field_name (key));
                       res++;
                       break;
                     }
                 }
 
-              rec_field_name_destroy (key_field_name);
+              free (key_field_name);
             }
         }                                          
     }
@@ -681,12 +686,10 @@ rec_int_check_descriptor (rec_rset_t rset,
   rec_record_t descriptor;
   rec_mset_iterator_t iter;
   rec_field_t field;
-  char *field_name_str;
-  rec_field_name_t field_name;
-  char *field_value;
+  const char *field_name;
+  const char *field_value;
   rec_fex_t fex;
-  rec_field_name_t auto_field_name;
-  char *auto_field_name_str;
+  const char *auto_field_name;
   size_t i;
   rec_type_t type;
   char *type_name = NULL;
@@ -767,7 +770,6 @@ rec_int_check_descriptor (rec_rset_t rset,
       while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void**) &field, NULL))
         {
           field_name = rec_field_name (field);
-          field_name_str = rec_field_name_str (field);
           field_value = rec_field_value (field);
 
           if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_TYPE)))
@@ -877,10 +879,40 @@ does not exist\n"),
                     }
                 }
             }
+          else if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_CONSTRAINT)))
+            {
+              /* Check that the value of this field is a valid
+                 selection expression.  */
+              
+              rec_sex_t sex = rec_sex_new (false);
+              if (sex)
+                {
+                  if (rec_sex_compile (sex, field_value))
+                    {
+                      rec_sex_destroy (sex);
+                    }
+                  else
+                    {
+                      ADD_ERROR (errors,
+                                 _("%s:%s: error: value for %s[%zd] is not a valid selection expression\n"),
+                                 rec_record_source (descriptor),
+                                 rec_record_location_str (descriptor),
+                                 rec_field_name (field),
+                                 rec_record_get_field_index_by_name (descriptor, field));
+                      res++;
+                    }
+                }
+              else
+                {
+                  /* Out of memory.  */
+                  res++;
+                }
+            }
           else if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_MANDATORY))
                    || rec_field_name_equal_p (field_name, FNAME(REC_FIELD_UNIQUE))
                    || rec_field_name_equal_p (field_name, FNAME(REC_FIELD_PROHIBIT))
-                   || rec_field_name_equal_p (field_name, FNAME(REC_FIELD_AUTO)))
+                   || rec_field_name_equal_p (field_name, FNAME(REC_FIELD_AUTO))
+                   || rec_field_name_equal_p (field_name, FNAME(REC_FIELD_SORT)))
             {
               /* Check that the value of this field is a parseable
                  list of field names.  */
@@ -895,35 +927,20 @@ does not exist\n"),
                              _("%s:%s: error: value for %s[%zd] is not a list of field names\n"),
                              rec_record_source (descriptor),
                              rec_record_location_str (descriptor),
-                             rec_field_name_str (field),
+                             rec_field_name (field),
                              rec_record_get_field_index_by_name (descriptor, field));
                   res++;
                 }
             }
-          else if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_SORT)))
-            {
-              if (!rec_match (field_value,
-                              "^"
-                              "[ \n\t]*" REC_FNAME_RE "[ \n\t]*"
-                              "$"))
-                {
-                  ADD_ERROR (errors,
-                            _("%s:%s: error: value for %s shall be a field name.\n"),
-                            rec_field_source (field),
-                            rec_field_location_str (field),
-                            field_name_str);
-                  res++ ;
-                }
-            }
           else if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_SIZE)))
             {
               if (!rec_match (field_value, REC_INT_SIZE_RE))
                 {
                   ADD_ERROR (errors,
-                            _("%s:%s: error: value for %s shall be a number optionally preceded by >, <, >= or <=.\n"),
+                            _("%s:%s: error: value for %s should be a number optionally preceded by >, <, >= or <=.\n"),
                             rec_field_source (field),
                             rec_field_location_str (field),
-                            field_name_str);
+                            field_name);
                   res++;
                 }
             }
@@ -936,10 +953,10 @@ does not exist\n"),
                               "[ \n\t]*$"))
                 {
                   ADD_ERROR (errors,
-                            _("%s:%s: error: value for %s shall be a list of field names.\n"),
+                            _("%s:%s: error: value for %s should be a list of field names.\n"),
                             rec_field_source (field),
                             rec_field_location_str (field),
-                            field_name_str);
+                            field_name);
                   res++;
                 }
             }
@@ -953,18 +970,24 @@ does not exist\n"),
               for (i = 0; i < rec_fex_size (fex); i++)
                 {
                   auto_field_name = rec_fex_elem_field_name (rec_fex_get (fex, i));
-                  auto_field_name_str = rec_fex_elem_field_name_str (rec_fex_get (fex, i));
                   type = rec_rset_get_field_type (rset, auto_field_name);
                   if ((!type) ||
                       ! ((rec_type_kind (type) == REC_TYPE_INT)
                          || (rec_type_kind (type) == REC_TYPE_RANGE)
+#if defined UUID_TYPE
+                         || (rec_type_kind (type) == REC_TYPE_UUID)
+#endif
                          || (rec_type_kind (type) == REC_TYPE_DATE)))
                     {
                       ADD_ERROR (errors,
-                                _("%s:%s: error: auto-incremented field %s shall be of type int, range or date\n"),
+#if defined UUID_TYPE
+                                 _("%s:%s: error: auto-incremented field %s should be of type int, range, uuid or date\n"),
+#else
+                                 _("%s:%s: error: auto-incremented field %s should be of type int, range or date\n"),
+#endif
                                 rec_record_source (descriptor),
                                 rec_record_location_str (descriptor),
-                                auto_field_name_str);
+                                auto_field_name);
                       res++;
                     }
                 }
@@ -1031,7 +1054,7 @@ rec_int_merge_remote (rec_rset_t rset,
               curl = curl_easy_init ();
 
               /* Create a temporary file.  */
-              strncpy (tmpfile_name, "recint-XXXXXX", 13);
+              memcpy (tmpfile_name, "recint-XXXXXX", 13);
               tmpfile_name[13] = '\0';
               tmpfile_des = gen_tempname (tmpfile_name, 0, 0, GT_FILE);
               external_file = fdopen (tmpfile_des, "r+");
@@ -1141,11 +1164,11 @@ rec_int_merge_remote (rec_rset_t rset,
 }
 
 static bool
-rec_int_rec_type_p (char *str)
+rec_int_rec_type_p (const char *str)
 {
   return rec_match (str,
                     "^[ \t]*"
-                    REC_FNAME_PART_RE
+                    REC_RECORD_TYPE_RE
                     "[ \n\t]*"
                     "("
                     "(" REC_URL_REGEXP ")"
