@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011, 2012 Jose E. Marchesi */
+/* Copyright (C) 2009, 2010, 2011, 2012, 2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,16 +41,15 @@
 static int rec_parser_getc (rec_parser_t parser);
 static int rec_parser_ungetc (rec_parser_t parser, int ci);
 
-static bool rec_expect (rec_parser_t parser, char *str);
+static bool rec_expect (rec_parser_t parser, const char *str);
 
-static bool rec_parse_field_name_part (rec_parser_t parser, char **str);
 static bool rec_parse_field_value (rec_parser_t parser, char **str);
 
 static bool rec_parse_comment (rec_parser_t parser, rec_comment_t *comment);
 
 static bool rec_parser_digit_p (char c);
 static bool rec_parser_letter_p (char c);
-static bool rec_parser_init_common (rec_parser_t parser, char *source);
+static bool rec_parser_init_common (rec_parser_t parser, const char *source);
 
 /*
  * Parser Data Structure
@@ -71,10 +70,11 @@ enum rec_parser_error_e
 
 struct rec_parser_s
 {
-  FILE *in_file;     /* File stream used by the parser.  */
-  char *in_buffer;   /* Buffer used by the parser.  */
-  char *p;           /* Pointer to the next unreaded character in
-                        in_buffer */
+  FILE *in_file;          /* File stream used by the parser.  */
+  const char *in_buffer;  /* Buffer used by the parser.  */
+  size_t in_size;         /* Length of in_buffer. */
+  const char *p;          /* Pointer to the next unreaded character in
+                             in_buffer */
   char *source;
   
   rec_record_t prev_descriptor;
@@ -112,7 +112,7 @@ const char *rec_parser_error_strings[] =
 
 rec_parser_t
 rec_parser_new (FILE *in,
-                char *source)
+                const char *source)
 {
   rec_parser_t parser;
 
@@ -121,6 +121,7 @@ rec_parser_new (FILE *in,
     {
       parser->in_file = in;
       parser->in_buffer = NULL;
+      parser->in_size = 0;
 
       if (!rec_parser_init_common (parser, source))
         {
@@ -133,8 +134,16 @@ rec_parser_new (FILE *in,
 }
 
 rec_parser_t
-rec_parser_new_str (char *buffer,
-                    char *source)
+rec_parser_new_str (const char *buffer,
+                    const char *source)
+{
+  return rec_parser_new_mem (buffer, strlen (buffer), source);
+}
+
+rec_parser_t
+rec_parser_new_mem (const char *buffer,
+                    size_t size,
+                    const char *source)
 {
   rec_parser_t parser;
 
@@ -142,6 +151,7 @@ rec_parser_new_str (char *buffer,
   if (parser != NULL)
     {
       parser->in_buffer = buffer;
+      parser->in_size = size;
       parser->in_file = NULL;
 
       if (!rec_parser_init_common (parser, source))
@@ -157,8 +167,11 @@ rec_parser_new_str (char *buffer,
 void
 rec_parser_destroy (rec_parser_t parser)
 {
-  free (parser->source);
-  free (parser);
+  if (parser)
+    {
+      free (parser->source);
+      free (parser);
+    }
 }
 
 bool
@@ -175,7 +188,7 @@ rec_parser_error (rec_parser_t parser)
 
 void
 rec_parser_perror (rec_parser_t parser,
-                   char *fmt,
+                   const char *fmt,
                    ...)
 {
   va_list ap;
@@ -184,7 +197,7 @@ rec_parser_perror (rec_parser_t parser,
   va_start (ap, fmt);
   vfprintf (stderr, fmt, ap);
   fputs (": ", stderr);
-  number_str = malloc(10);
+  number_str = NULL;  /* asprintf does the allocation */
   if (asprintf (&number_str, "%zu", parser->line) != -1)
     {
       fputs (number_str, stderr);
@@ -199,12 +212,13 @@ rec_parser_perror (rec_parser_t parser,
 
 bool
 rec_parse_field_name (rec_parser_t parser,
-                      rec_field_name_t *fname)
+                      char **fname)
 {
-  bool ret;
-  char *name_part;
+  bool ret = true;
   int ci;
+  size_t str_size;
   char c;
+  rec_buf_t buf;
 
   /* Sanity check */
   if (rec_parser_eof (parser)
@@ -213,78 +227,130 @@ rec_parse_field_name (rec_parser_t parser,
       return false;
     }
 
-  *fname = rec_field_name_new ();
-  if (*fname == NULL)
+  buf = rec_buf_new (fname, &str_size);
+  if (!buf)
     {
-      /* End of memory */
+      /* Out of memory */
       parser->error = REC_PARSER_ENOMEM;
       return false;
     }
 
-  ret = false;
-  while (rec_parse_field_name_part (parser, &name_part))
-    {
-      if ((parser->eof)
-          || (parser->error > 0))
-        {
-          break;
-        }
+  /* The syntax of a field name is described by the following regexp:
+   *
+   * [a-zA-Z%][a-zA-Z0-9_]*:
+   */
 
-      /* Add this name part to the name */
-      if (!rec_field_name_set (*fname,
-                               rec_field_name_size (*fname),
-                               name_part))
+  /* [a-zA-Z%] */
+  ci = rec_parser_getc (parser);
+  if (ci == EOF)
+    {
+      ret = false;
+    }
+  else
+    {
+      c = (char) ci;
+
+      if ((rec_parser_letter_p (c))
+          || (c == '%'))
         {
-          /* Too much parts */
-          parser->error = REC_PARSER_ETOOMUCHNAMEPARTS;
+          if (rec_buf_putc (c, buf) == EOF)
+            {
+              /* Out of memory */
+              parser->error = REC_PARSER_ENOMEM;
+              return false;
+            }
         }
       else
         {
-          ret = true;
-
-          /* Field names ends with:
-           *
-           * - A blank character or
-           * - A newline or
-           * - The end of the file
-           *
-           * Note that if the field name ends with a newline it is
-           * pushed back to the input stream, since (unlike a blank
-           * character) it will be part of the field value.
-           */
-          ci = rec_parser_getc (parser);
-          if (ci != EOF)
-            {
-              c = (char) ci;
-              if (c == ' ')
-                {
-                  parser->error = REC_PARSER_NOERROR;
-                  break;
-                }
-              else if (c == '\n')
-                {
-                  parser->error = REC_PARSER_NOERROR;
-                  rec_parser_ungetc (parser, c);
-                  break;
-                }
-              else
-                {
-                  rec_parser_ungetc (parser, c);
-                }
-            }
-          else
-            {
-              break;
-            }
+          /* Parse error */
+          parser->error = REC_PARSER_EFNAME;
+          ret = false;
         }
     }
 
-  if (!ret)
+  /* [a-zA-Z0-9_]* */
+  if (ret)
     {
-      /* Free used memory */
-      rec_field_name_destroy (*fname);
+      while ((ci = rec_parser_getc (parser)) != EOF)
+        {
+          c = (char) ci;
+
+          if (rec_parser_letter_p (c)
+              || rec_parser_digit_p (c)
+              || (c == '_'))
+            {
+              if (rec_buf_putc (c, buf) == EOF)
+                {
+                  /* Out of memory */
+                  parser->error = REC_PARSER_ENOMEM;
+                  return false;
+                }
+              if (parser->error > 0)
+                {
+                  /* error was set by ADD_TO_STR */
+                  break;
+                }
+            }
+          else if (c == ':')
+            {
+              /* End of token.  Consume the ':' and report success */
+              break;
+            }
+          else
+            {
+              /* Parse error */
+              parser->error = REC_PARSER_EFNAME;
+              ret = false;
+              break;
+            }
+        }
+
+      if (parser->eof)
+        {
+          parser->error = REC_PARSER_EFNAME;
+          ret = false;
+        }
     }
 
+  rec_buf_close (buf);
+
+  if (!ret)
+    {
+      free (*fname);
+    }
+  else
+    {
+      /* Field names ends with:
+       *
+       * - A blank character or
+       * - A newline or
+       * - The end of the file
+       *
+       * Note that if the field name ends with a newline it is
+       * pushed back to the input stream, since (unlike a blank
+       * character) it will be part of the field value.
+       */
+
+      ci = rec_parser_getc (parser);
+      if (ci != EOF)
+        {
+          c = (char) ci;
+          if (c == ' ')
+            {
+              parser->error = REC_PARSER_NOERROR;
+            }
+          else if (c == '\n')
+            {
+              parser->error = REC_PARSER_NOERROR;
+              rec_parser_ungetc (parser, c);
+            }
+          else
+            {
+              rec_parser_ungetc (parser, c);
+            }
+        }
+    }
+  
   return ret;
 }
 
@@ -302,7 +368,7 @@ rec_parse_field (rec_parser_t parser,
 {
   bool ret;
   rec_field_t new;
-  rec_field_name_t field_name;
+  char *field_name;
   char *field_value;
   size_t location;
   size_t char_location;
@@ -330,16 +396,18 @@ rec_parse_field (rec_parser_t parser,
         {
           new = rec_field_new (field_name,
                                field_value);
+	  if (new == NULL)
+	    return false;
 
           rec_field_set_source (new, parser->source);
           rec_field_set_location (new, location);
           rec_field_set_char_location (new, char_location);
           *field = new;
+
+          free (field_value);
         }
-      else
-        {
-          rec_field_name_destroy (field_name);
-        }
+
+      free (field_name);
     }
 
   return ret;
@@ -395,6 +463,8 @@ rec_parse_record (rec_parser_t parser,
     {
       /* Expected a field.  */
       parser->error = REC_PARSER_EFIELD;
+      rec_record_destroy (new);
+      *record = NULL;
       return false;
     }
 
@@ -412,6 +482,30 @@ rec_parse_record (rec_parser_t parser,
               rec_mset_append (rec_record_mset (new), MSET_COMMENT, (void *) comment, MSET_ANY);
             }
         }
+      else if ((c == ' ') || (c == '\t'))
+        {
+          /* A line composed just by blanks acts like an end of record
+             separator. */
+
+          while ((ci != EOF) && ((c == ' ') || (c == '\t')))
+            {
+              ci = rec_parser_getc (parser);
+              c = (char) ci;
+            }
+
+          if ((ci == EOF) || (c == '\n'))
+            {
+              /* End of record */
+              break;
+            }
+          else
+            {
+              /* Parse error: field expected */
+              parser->error = REC_PARSER_EFIELD;
+              ret = false;
+              break;
+            }
+        }
       else if (c == '\n')
         {
           /* End of record */
@@ -592,6 +686,11 @@ rec_parse_db (rec_parser_t parser,
         }
     }
 
+  if (!rec_parser_eof (parser))
+    {
+      ret = false;
+    }
+
   if (ret)
     {
       *db = new;
@@ -600,11 +699,11 @@ rec_parse_db (rec_parser_t parser,
   return ret;
 }
 
-rec_field_name_t
-rec_parse_field_name_str (char *str)
+char *
+rec_parse_field_name_str (const char *str)
 {
   rec_parser_t parser;
-  rec_field_name_t field_name;
+  char *field_name = NULL;
   char *str2;
   size_t str_size;
 
@@ -617,8 +716,8 @@ rec_parse_field_name_str (char *str)
       return NULL;
     }
 
-  strncpy (str2, str, str_size);
-  if (str2[str_size - 1] == ':')
+  memcpy (str2, str, str_size);
+  if (str_size > 0 && str2[str_size - 1] == ':')
     {
       str2[str_size] = '\0';
     }
@@ -628,8 +727,6 @@ rec_parse_field_name_str (char *str)
       str2[str_size + 1] = '\0';
     }
   
-  field_name = NULL;
-
   parser = rec_parser_new_str (str2, "dummy");
   if (!rec_parse_field_name (parser, &field_name))
     {
@@ -639,21 +736,18 @@ rec_parse_field_name_str (char *str)
   if (!rec_parser_eof (parser))
     {
       /* There is additional stuff after the field name.  */
-      if (field_name)
-        {
-          rec_field_name_destroy (field_name);
-        }
+      free (field_name);
       field_name = NULL;
     }
 
   rec_parser_destroy (parser);
   free (str2);
-  
+
   return field_name;
 }
 
 rec_record_t
-rec_parse_record_str (char *str)
+rec_parse_record_str (const char *str)
 {
   rec_parser_t parser;
   rec_record_t record;
@@ -672,6 +766,58 @@ rec_parse_record_str (char *str)
   return record;
 }
 
+bool
+rec_parser_seek (rec_parser_t parser,
+                 size_t line_number,
+                 size_t position)
+{
+  if (parser->in_file)
+    {
+      if (fseek (parser->in_file, position, SEEK_SET))
+        {
+          return false;
+        }
+    }
+  else if (parser->in_buffer)
+    {
+      if (position > parser->in_size)
+        {
+          return false;
+        }
+      parser->p = parser->in_buffer + position;
+    }
+  else
+    {
+      /* This point should not be reached!  */
+      fprintf (stderr, "rec_parser_seek: no backend in parser. This is a bug.\
+  Please report it.");
+      return false;
+    }
+  parser->line = line_number;
+  parser->character = position;
+  return true;
+}
+
+long
+rec_parser_tell (rec_parser_t parser)
+{
+  if (parser->in_file)
+    {
+      return ftell (parser->in_file);
+    }
+  else if (parser->in_buffer)
+    {
+      return parser->p - parser->in_buffer;
+    }
+  else
+    {
+      /* This point should not be reached!  */
+      fprintf (stderr, "rec_parser_seek: no backend in parser. This is a bug.\
+  Please report it.");
+      return -1;
+    }
+}
+
 /*
  * Private functions
  */
@@ -689,7 +835,7 @@ rec_parser_getc (rec_parser_t parser)
     }
   else if (parser->in_buffer)
     {
-      if (*(parser->p) == '\0')
+      if (parser->p == parser->in_buffer + parser->in_size)
         {
           ci = EOF;
         }
@@ -790,7 +936,7 @@ rec_parser_letter_p (char c)
 
 static bool
 rec_expect (rec_parser_t parser,
-            char *str)
+            const char *str)
 {
   size_t str_size;
   size_t counter;
@@ -829,113 +975,6 @@ rec_expect (rec_parser_t parser,
   return found;
 }
 
-static bool
-rec_parse_field_name_part (rec_parser_t parser,
-                           char **str)
-{
-  bool ret;
-  int ci;
-  size_t str_size;
-  char c;
-  rec_buf_t buf;
-
-  ret = true;
-  buf = rec_buf_new (str, &str_size);
-  if (!buf)
-    {
-      /* Out of memory */
-      parser->error = REC_PARSER_ENOMEM;
-      return false;
-    }
-
-  /* The syntax of a field name is described by the following regexp:
-   *
-   * [a-zA-Z%][a-zA-Z0-9_]*
-   */
-
-  /* [a-zA-Z%] */
-  ci = rec_parser_getc (parser);
-  if (ci == EOF)
-    {
-      ret = false;
-    }
-  else
-    {
-      c = (char) ci;
-
-      if ((rec_parser_letter_p (c))
-          || (c == '%'))
-        {
-          if (rec_buf_putc (c, buf) == EOF)
-            {
-              /* Out of memory */
-              parser->error = REC_PARSER_ENOMEM;
-              return false;
-            }
-        }
-      else
-        {
-          /* Parse error */
-          parser->error = REC_PARSER_EFNAME;
-          ret = false;
-        }
-    }
-
-  /* [a-zA-Z0-9_]* */
-  if (ret)
-    {
-      while ((ci = rec_parser_getc (parser)) != EOF)
-        {
-          c = (char) ci;
-
-          if (rec_parser_letter_p (c)
-              || rec_parser_digit_p (c)
-              || (c == '-')
-              || (c == '_'))
-            {
-              if (rec_buf_putc (c, buf) == EOF)
-                {
-                  /* Out of memory */
-                  parser->error = REC_PARSER_ENOMEM;
-                  return false;
-                }
-              if (parser->error > 0)
-                {
-                  /* error was set by ADD_TO_STR */
-                  break;
-                }
-            }
-          else if (c == ':')
-            {
-              /* End of token.  Consume the ':' and report success */
-              break;
-            }
-          else
-            {
-              /* Parse error */
-              parser->error = REC_PARSER_EFNAME;
-              ret = false;
-              break;
-            }
-        }
-
-      if (parser->eof)
-        {
-          parser->error = REC_PARSER_EFNAME;
-          ret = false;
-        }
-    }
-
-  rec_buf_close (buf);
-
-  if (!ret)
-    {
-      free (*str);
-    }
-
-  return ret;
-}
-
 static bool
 rec_parse_field_value (rec_parser_t parser,
                        char **str)
@@ -1184,16 +1223,16 @@ rec_parse_comment (rec_parser_t parser, rec_comment_t *comment)
     }
   else
     {
-      free (str);
       *comment = NULL;
     }
 
+  free (str);
   return ret;
 }
 
 static bool
 rec_parser_init_common (rec_parser_t parser,
-                        char *source)
+                        const char *source)
 {
   if (source)
     {
