@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010 Jose E. Marchesi */
+/* Copyright (C) 2010-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,11 +46,14 @@ enum rec_sex_ast_node_type_e
   REC_SEX_OP_MAT,
   REC_SEX_OP_LT,
   REC_SEX_OP_GT,
+  REC_SEX_OP_LTE,
+  REC_SEX_OP_GTE,
   REC_SEX_OP_AND,
   REC_SEX_OP_OR,
   REC_SEX_OP_NOT,
   REC_SEX_OP_SHA,
   REC_SEX_OP_SAMETIME,
+  REC_SEX_OP_IMPLIES,
   REC_SEX_OP_BEFORE,
   REC_SEX_OP_AFTER,
   REC_SEX_OP_COND,
@@ -95,8 +98,9 @@ double rec_sex_ast_node_real (rec_sex_ast_node_t node);
 void rec_sex_ast_node_set_real (rec_sex_ast_node_t node, double num);
 char *rec_sex_ast_node_str (rec_sex_ast_node_t node);
 void rec_sex_ast_node_set_str (rec_sex_ast_node_t node, char *str);
-char *rec_sex_ast_node_name (rec_sex_ast_node_t node);
-void rec_sex_ast_node_set_name (rec_sex_ast_node_t node, char *name);
+const char *rec_sex_ast_node_name (rec_sex_ast_node_t node);
+const char *rec_sex_ast_node_subname (rec_sex_ast_node_t node);
+void rec_sex_ast_node_set_name (rec_sex_ast_node_t node, const char *name, const char *subname);
 
 int rec_sex_ast_node_num_children (rec_sex_ast_node_t node);
 
@@ -118,6 +122,10 @@ void rec_sex_ast_node_unfix (rec_sex_ast_node_t node);
 bool rec_sex_ast_node_fixed (rec_sex_ast_node_t node);
 char *rec_sex_ast_node_fixed_val (rec_sex_ast_node_t node);
 
+/* This function returns 'true' if there is a node on AST of type
+   REC_SEX_NAME where NAME.name == NAME and NAME.idx <= IDX.  */
+bool rec_sex_ast_name_p (rec_sex_ast_t ast, const char *name, size_t idx);
+
 #endif /* rec-sex-ast.h */
 
 
