@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011, 2012 Jose E. Marchesi */
+/* Copyright (C) 2010-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,11 +29,16 @@
 #include <stdio.h>
 #include <regex.h>
 #include <string.h>
+#include <limits.h>
 #include <regex.h>
 #include <parse-datetime.h>
 #include <gettext.h>
 #define _(str) dgettext (PACKAGE, str)
 
+#if defined UUID_TYPE
+#  include <uuid/uuid.h>
+#endif
+
 #include <rec-utils.h>
 #include <rec.h>
 
@@ -43,6 +48,7 @@
 
 /* Textual name of the types expected in the type description
    strings.  */
+
 #define REC_TYPE_INT_NAME    "int"
 #define REC_TYPE_BOOL_NAME   "bool"
 #define REC_TYPE_RANGE_NAME  "range"
@@ -54,6 +60,11 @@
 #define REC_TYPE_ENUM_NAME   "enum"
 #define REC_TYPE_EMAIL_NAME  "email"
 #define REC_TYPE_FIELD_NAME  "field"
+#define REC_TYPE_REC_NAME    "rec"
+
+#if defined UUID_TYPE
+#  define REC_TYPE_UUID_NAME   "uuid"
+#endif
 
 /* Regular expressions.  */
 
@@ -67,8 +78,13 @@
 #define REC_TYPE_INT_VALUE_RE                   \
   "^" REC_TYPE_ZBLANKS_RE REC_INT_RE REC_TYPE_ZBLANKS_RE "$"
 
+#define REC_TYPE_BOOL_TRUE_VALUES_RE  "1|yes|true"
+#define REC_TYPE_BOOL_FALSE_VALUES_RE "0|no|false"
+
 #define REC_TYPE_BOOL_VALUE_RE                  \
-  "^" REC_TYPE_ZBLANKS_RE "(yes|no|true|false|0|1)" REC_TYPE_ZBLANKS_RE "$"
+  "^" REC_TYPE_ZBLANKS_RE "(" REC_TYPE_BOOL_TRUE_VALUES_RE "|" \
+                              REC_TYPE_BOOL_FALSE_VALUES_RE    \
+                          ")" REC_TYPE_ZBLANKS_RE "$"
 
 #define REC_TYPE_REAL_VALUE_RE                  \
   "^" REC_TYPE_ZBLANKS_RE "-?([0-9]+)?(\\.[0-9]+)?" REC_TYPE_ZBLANKS_RE "$"
@@ -98,14 +114,24 @@
   REC_TYPE_ZBLANKS_RE                           \
   "$"
   
+#define REC_TYPE_REC_VALUE_RE
+
 /* Regular expression denoting a type name.  */
+
+#if defined UUID_TYPE
+#  define REC_TYPE_CLASS_UUID_RE "|" REC_TYPE_UUID_NAME
+#else
+#  define REC_TYPE_CLASS_UUID_RE
+#endif
+
 #define REC_TYPE_CLASS_RE                               \
   "(" REC_TYPE_INT_NAME   "|" REC_TYPE_RANGE_NAME  "|" \
       REC_TYPE_REAL_NAME  "|" REC_TYPE_SIZE_NAME   "|" \
       REC_TYPE_LINE_NAME  "|" REC_TYPE_REGEXP_NAME "|" \
       REC_TYPE_DATE_NAME  "|" REC_TYPE_ENUM_NAME   "|" \
       REC_TYPE_EMAIL_NAME "|" REC_TYPE_BOOL_NAME   "|" \
-      REC_TYPE_FIELD_NAME \
+      REC_TYPE_FIELD_NAME "|" REC_TYPE_REC_NAME        \
+      REC_TYPE_CLASS_UUID_RE                           \
   ")"
 
 /* Regular expressions for the type descriptions.  */
@@ -119,13 +145,14 @@
   REC_TYPE_BOOL_NAME
 
 /* range MIN MAX  */
+#define REC_TYPE_RANGE_MINMAX_RE "MIN | MAX"
 #define REC_TYPE_RANGE_DESCR_RE                    \
   REC_TYPE_RANGE_NAME                              \
   REC_TYPE_BLANKS_RE                               \
-  REC_INT_RE                                       \
+  "(" REC_INT_RE "|" REC_TYPE_RANGE_MINMAX_RE ")"  \
   "("                                              \
-  REC_TYPE_ZBLANKS_RE                              \
-  REC_INT_RE                                       \
+     REC_TYPE_ZBLANKS_RE                           \
+     "(" REC_INT_RE "|" REC_TYPE_RANGE_MINMAX_RE ")"\
   ")?"
 
 /* real  */
@@ -158,10 +185,14 @@
   REC_TYPE_ENUM_NAME_RE                         \
   "(" REC_TYPE_BLANKS_RE REC_TYPE_ENUM_NAME_RE ")*"
 
-  /* field */
+/* field */
 #define REC_TYPE_FIELD_DESCR_RE                 \
   REC_TYPE_FIELD_NAME
 
+/* rec RECORD_TYPE */
+#define REC_TYPE_REC_DESCR_RE                   \
+  REC_TYPE_REC_NAME REC_TYPE_BLANKS_RE REC_RECORD_TYPE_RE
+
 /* email */
 #define REC_TYPE_EMAIL_DESCR_RE                 \
   REC_TYPE_EMAIL_NAME
@@ -178,13 +209,13 @@
      "|" "(" REC_TYPE_RANGE_DESCR_RE  ")"       \
      "|" "(" REC_TYPE_REAL_DESCR_RE   ")"       \
      "|" "(" REC_TYPE_SIZE_DESCR_RE   ")"       \
-     "|" "(" REC_TYPE_RANGE_DESCR_RE  ")"       \
      "|" "(" REC_TYPE_LINE_DESCR_RE   ")"       \
      "|" "(" REC_TYPE_REGEXP_DESCR_RE ")"       \
      "|" "(" REC_TYPE_DATE_DESCR_RE   ")"       \
      "|" "(" REC_TYPE_EMAIL_DESCR_RE  ")"       \
      "|" "(" REC_TYPE_ENUM_DESCR_RE   ")"       \
      "|" "(" REC_TYPE_FIELD_DESCR_RE  ")"       \
+     "|" "(" REC_TYPE_REC_DESCR_RE    ")"       \
   ")"                                           \
   REC_TYPE_ZBLANKS_RE                           \
   "$"
@@ -200,18 +231,22 @@ struct rec_type_s
   enum rec_type_kind_e kind;  /* Kind of the type.  */
   char *expr;                 /* Copy of the type descriptor used to
                                  create the type.  */
+  size_t size;                /* Used for enumerations: number of
+                                 names.  */
+
   union
   {
     size_t max_size;          /* Size of string.  */
     int range[2];             /* Range.  */
     regex_t regexp;           /* Regular expression.  */
+    char *recname;            /* Record.  */
 
-#define REC_ENUM_MAX_NAMES 50
-    char *names[REC_ENUM_MAX_NAMES];   /* Names in enumeration.  */
+#define REC_ENUM_ALLOC_NAMES 50
+    char **names;             /* Names in enumeration.  */
   } data;
 };
 
-#define REC_TYPE_REG_MAX_TYPES  100
+#define REC_TYPE_REG_ALLOC_TYPES 100
 
 struct rec_type_reg_entry_s
 {
@@ -225,7 +260,7 @@ struct rec_type_reg_entry_s
 struct rec_type_reg_s
 {
   size_t num_types;
-  struct rec_type_reg_entry_s types[REC_TYPE_REG_MAX_TYPES];
+  struct rec_type_reg_entry_s *types;
 };
 
 /*
@@ -234,17 +269,22 @@ struct rec_type_reg_s
 
 static enum rec_type_kind_e rec_type_parse_type_kind (char *str);
 
-static bool rec_type_check_int (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_bool (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_range (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_real (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_size (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_line (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_regexp (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_date (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_email (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_enum (rec_type_t type, char *str, rec_buf_t errors);
-static bool rec_type_check_field (rec_type_t type, char *str, rec_buf_t errors);
+static bool rec_type_check_int (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_bool (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_range (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_real (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_size (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_line (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_regexp (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_date (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_email (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_enum (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_field (rec_type_t type, const char *str, rec_buf_t errors);
+static bool rec_type_check_rec (rec_type_t type, const char *str, rec_buf_t errors);
+
+#if defined UUID_TYPE
+static bool rec_type_check_uuid (rec_type_t type, const char *str, rec_buf_t errors);
+#endif
 
 /* Parsing routines.  */
 
@@ -252,6 +292,7 @@ static const char *rec_type_parse_size (const char *str, rec_type_t type);
 static const char *rec_type_parse_enum (const char *str, rec_type_t type);
 static const char *rec_type_parse_regexp_type (const char *str, rec_type_t type);
 static const char *rec_type_parse_range (const char *str, rec_type_t type);
+static const char *rec_type_parse_rec (const char *str, rec_type_t type);
 
 /*
  * Public functions.
@@ -320,6 +361,7 @@ rec_type_new (const char *str)
       goto exit;
     }
   new->name = NULL; /* Newly created types are anonyous. */
+  new->size = 0;
 
   rec_skip_blanks (&p);
 
@@ -375,6 +417,16 @@ rec_type_new (const char *str)
           }
         break;
       }
+    case REC_TYPE_REC:
+      {
+        p = rec_type_parse_rec (p, new);
+        if (!p)
+          {
+            free (new);
+            new = NULL;
+          }
+        break;
+      }
     case REC_TYPE_INT:
     case REC_TYPE_BOOL:
     case REC_TYPE_REAL:
@@ -382,6 +434,9 @@ rec_type_new (const char *str)
     case REC_TYPE_FIELD:
     case REC_TYPE_DATE:
     case REC_TYPE_EMAIL:
+#if defined UUID_TYPE
+    case REC_TYPE_UUID:
+#endif
       {
         /* We are done.  */
         break;
@@ -404,7 +459,7 @@ rec_type_new (const char *str)
         {
           if (!rec_blank_p (*p))
             {
-              free (new);
+              rec_type_destroy (new);
               new = NULL;
               break;
             }
@@ -493,6 +548,18 @@ rec_type_kind_str (rec_type_t type)
         res = REC_TYPE_FIELD_NAME;
         break;
       }
+    case REC_TYPE_REC:
+      {
+        res = REC_TYPE_REC_NAME;
+        break;
+      }
+#if defined UUID_TYPE
+    case REC_TYPE_UUID:
+      {
+        res = REC_TYPE_UUID_NAME;
+        break;
+      }
+#endif
     default:
       {
         res = REC_TYPE_NONE;
@@ -505,7 +572,7 @@ rec_type_kind_str (rec_type_t type)
 
 bool
 rec_type_check (rec_type_t type,
-                char *str,
+                const char *str,
                 char **error_str)
 {
   bool res;
@@ -578,6 +645,18 @@ rec_type_check (rec_type_t type,
         res = rec_type_check_field (type, str, errors);
         break;
       }
+    case REC_TYPE_REC:
+      {
+        res = rec_type_check_rec (type, str, errors);
+        break;
+      }
+#if defined UUID_TYPE
+    case REC_TYPE_UUID:
+      {
+        res = rec_type_check_uuid (type, str, errors);
+        break;
+      }
+#endif
     }
 
   /* Terminate the 'errors' string.  */
@@ -599,8 +678,25 @@ rec_type_check (rec_type_t type,
 void
 rec_type_destroy (rec_type_t type)
 {
-  free (type->name);
-  free (type);
+  int i;
+
+  if (type)
+    {
+      if (type->kind == REC_TYPE_ENUM)
+        {
+          for (i = 0; i < type->size; i++)
+            {
+              free (type->data.names[i]);
+            }
+        }
+      else if (type->kind == REC_TYPE_REGEXP)
+        {
+          regfree (&type->data.regexp);
+        }
+      
+      free (type->name);
+      free (type);
+    }
 }
 
 rec_type_reg_t
@@ -612,6 +708,7 @@ rec_type_reg_new (void)
   if (new)
     {
       new->num_types = 0;
+      new->types = NULL;
     }
 
   return new;
@@ -631,6 +728,7 @@ rec_type_reg_destroy (rec_type_reg_t reg)
       free (reg->types[i].type_name);
       free (reg->types[i].to_type);
     }
+  free (reg->types);
   free (reg);
 }
 
@@ -663,15 +761,20 @@ rec_type_reg_add (rec_type_reg_t reg,
         }
     }
 
+  /* If we need to add a new entry then allocate it.  */
+
+  if (i == reg->num_types)
+    {
+      reg->types =
+        realloc (reg->types,
+                 ((i / REC_TYPE_REG_ALLOC_TYPES) + 1) * (sizeof (struct rec_type_reg_entry_s *) * REC_TYPE_REG_ALLOC_TYPES));
+      reg->num_types++;
+    }
+
   reg->types[i].type_name = strdup (rec_type_name (type));
   reg->types[i].type = type;
   reg->types[i].to_type = NULL;
   reg->types[i].visited_p = false;
-  if (i == reg->num_types)
-    {
-      /* We added a new entry.  */
-      reg->num_types++;
-    }
 }
 
 void
@@ -696,15 +799,21 @@ rec_type_reg_add_synonym (rec_type_reg_t reg,
         }
     }
 
+
+  /* If we need to add a new entry then allocate it.  */
+
+  if (i == reg->num_types)
+    {
+      reg->types =
+        realloc (reg->types,
+                 ((i / REC_TYPE_REG_ALLOC_TYPES) + 1) * (sizeof (struct rec_type_reg_entry_s *) * REC_TYPE_REG_ALLOC_TYPES));
+      reg->num_types++;
+    }
+
   reg->types[i].type_name = strdup (type_name);
   reg->types[i].to_type = strdup (to_type);
   reg->types[i].type = NULL;
   reg->types[i].visited_p = false;
-  if (i == reg->num_types)
-    {
-      /* We added a new entry.  */
-      reg->num_types++;
-    }
 }
 
 rec_type_t
@@ -786,14 +895,11 @@ rec_type_equal_p (rec_type_t type1,
         }
       else if (type1->kind == REC_TYPE_ENUM)
         {
-          i = 0;
-          while (type1->data.names[i])
+          for (i = 0; i < type1->size; i++)
             {
-              ret = (type2->data.names[i]
+              ret = ((i < type2->size)
                      && (strcmp (type1->data.names[i],
                                  type2->data.names[i]) == 0));
-
-              i++;
             }
         }
       else if (type1->kind == REC_TYPE_REGEXP)
@@ -841,6 +947,155 @@ rec_type_max (rec_type_t type)
   return res;
 }
 
+const char *
+rec_type_rec (rec_type_t type)
+{
+  const char *res = NULL;
+
+  if (type->kind == REC_TYPE_REC)
+    {
+      res = type->data.recname;
+    }
+
+  return res;
+}
+
+int
+rec_type_values_cmp (rec_type_t type,
+                     const char *val1,
+                     const char *val2)
+{
+  int type_comparison;
+  enum rec_type_kind_e kind = REC_TYPE_NONE;
+
+  if (type)
+    {
+      kind = type->kind;
+    }
+
+  switch (kind)
+    {
+    case REC_TYPE_INT:
+    case REC_TYPE_RANGE:
+      {
+        int int1, int2 = 0;
+
+        if (!rec_atoi (val1, &int1) || !rec_atoi (val2, &int2))
+          {
+            goto lexi;
+          }
+
+        if (int1 < int2)
+          {
+            type_comparison = -1;
+          }
+        else if (int1 > int2)
+          {
+            type_comparison = 1;
+          }
+        else
+          {
+            type_comparison = 0;
+          }
+
+        break;
+      }
+    case REC_TYPE_REAL:
+      {
+        double real1, real2 = 0;
+
+        if (!rec_atod (val1, &real1) || !rec_atod (val2, &real2))
+          {
+            goto lexi;
+          }
+
+        if (real1 < real2)
+          {
+            type_comparison = -1;
+          }
+        else if (real1 > real2)
+          {
+            type_comparison = 1;
+          }
+        else
+          {
+            type_comparison = 0;
+          }
+
+        break;
+      }
+    case REC_TYPE_BOOL:
+      {
+        bool bool1, bool2 = false;
+
+        /* Boolean fields storing 'false' come first.  */
+            
+        bool1 = rec_match (val1,
+                           REC_TYPE_ZBLANKS_RE "(" REC_TYPE_BOOL_TRUE_VALUES_RE ")" REC_TYPE_ZBLANKS_RE);
+        bool2 = rec_match (val2,
+                           REC_TYPE_ZBLANKS_RE "(" REC_TYPE_BOOL_TRUE_VALUES_RE ")" REC_TYPE_ZBLANKS_RE);
+
+        if (!bool1 && bool2)
+          {
+            type_comparison = -1;
+          }
+        else if (bool1 == bool2)
+          {
+            type_comparison = 0;
+          }
+        else
+          {
+            type_comparison = 1;
+          }
+
+        break;
+      }
+    case REC_TYPE_DATE:
+      {
+        struct timespec op1;
+        struct timespec op2;
+        struct timespec diff;
+
+        if (parse_datetime (&op1, val1, NULL)
+            && parse_datetime (&op2, val2, NULL))
+          {
+            if ((op1.tv_sec == op2.tv_sec)
+                && (op1.tv_nsec == op2.tv_nsec))
+              {
+                /* op1 == op2 */
+                type_comparison = 0;
+              }
+            else if (rec_timespec_subtract (&diff, &op1, &op2))
+              {
+                /* op1 < op2 */
+                type_comparison = -1;
+              }
+            else
+              {
+                /* op1 > op2 */
+                type_comparison = 1;
+              }
+          }
+        else
+          {
+            /* Invalid date => lexicographic order.  */
+            goto lexi;
+          }
+
+        break;
+      }
+    default:
+          {
+          lexi:
+            /* Lexicographic order.  */
+            type_comparison = strcmp (val1, val2);
+            break;
+          }
+    }
+
+  return type_comparison;
+}
+
 /*
  * Private functions.
  */
@@ -896,13 +1151,23 @@ rec_type_parse_type_kind (char *str)
     {
       res = REC_TYPE_FIELD;
     }
+  if (strcmp (str, REC_TYPE_REC_NAME) == 0)
+    {
+      res = REC_TYPE_REC;
+    }
+#if defined UUID_TYPE
+  if (strcmp (str, REC_TYPE_UUID_NAME) == 0)
+    {
+      res = REC_TYPE_UUID;
+    }
+#endif
 
   return res;
 }
 
 static bool
 rec_type_check_int (rec_type_t type,
-                    char *str,
+                    const char *str,
                     rec_buf_t errors)
 {
   bool ret;
@@ -916,9 +1181,23 @@ rec_type_check_int (rec_type_t type,
   return ret;
 }
 
+static bool
+rec_type_check_rec (rec_type_t type,
+                    const char *str,
+                    rec_buf_t errors)
+{
+  /* The values of type 'rec' are of whatever type the primary key of
+     the referred type is.  That check is not implemented here, but in
+     rec-types.c.
+
+     So sorry Mario, but the Princess is kept in another castle...  */
+  
+  return true;
+}
+
 static bool
 rec_type_check_field (rec_type_t type,
-                      char *str,
+                      const char *str,
                       rec_buf_t errors)
 {
   bool ret;
@@ -932,9 +1211,34 @@ rec_type_check_field (rec_type_t type,
   return ret;
 }
 
+#if defined UUID_TYPE
+
+static bool
+rec_type_check_uuid (rec_type_t type,
+                     const char *str,
+                     rec_buf_t errors)
+{
+  int ret;
+  uuid_t uu;
+
+  /* Determine whether the given string is a valid UUID by parsing it
+     using the uuid_parse function provided by the libuuid
+     library.  */
+
+  ret = uuid_parse (str, uu);
+  if ((ret == -1) && errors)
+    {
+      rec_buf_puts (_("invalid 'uuid' value."), errors);
+    }
+
+  return (ret == 0);
+}
+
+#endif /* UUID_TYPE */
+
 static bool
 rec_type_check_bool (rec_type_t type,
-                     char *str,
+                     const char *str,
                      rec_buf_t errors)
 {
   bool ret;
@@ -950,7 +1254,7 @@ rec_type_check_bool (rec_type_t type,
 
 static bool
 rec_type_check_range (rec_type_t type,
-                      char *str,
+                      const char *str,
                       rec_buf_t errors)
 {
   bool ret;
@@ -987,7 +1291,7 @@ rec_type_check_range (rec_type_t type,
 
 static bool
 rec_type_check_real (rec_type_t type,
-                     char *str,
+                     const char *str,
                      rec_buf_t errors)
 {
   bool ret;
@@ -1003,7 +1307,7 @@ rec_type_check_real (rec_type_t type,
 
 static bool
 rec_type_check_size (rec_type_t type,
-                     char *str,
+                     const char *str,
                      rec_buf_t errors)
 {
   bool ret;
@@ -1026,7 +1330,7 @@ rec_type_check_size (rec_type_t type,
 
 static bool
 rec_type_check_line (rec_type_t type,
-                     char *str, 
+                     const char *str, 
                      rec_buf_t errors)
 {
   bool ret;
@@ -1042,7 +1346,7 @@ rec_type_check_line (rec_type_t type,
 
 static bool
 rec_type_check_regexp (rec_type_t type,
-                       char *str,
+                       const char *str,
                        rec_buf_t errors)
 {
   bool ret;
@@ -1061,7 +1365,7 @@ rec_type_check_regexp (rec_type_t type,
 
 static bool
 rec_type_check_date (rec_type_t type,
-                     char *str,
+                     const char *str,
                      rec_buf_t errors)
 {
   bool ret;
@@ -1084,7 +1388,7 @@ rec_type_check_date (rec_type_t type,
 
 static bool
 rec_type_check_email (rec_type_t type,
-                      char *str,
+                      const char *str,
                       rec_buf_t errors)
 {
   bool ret;
@@ -1100,48 +1404,39 @@ rec_type_check_email (rec_type_t type,
 
 static bool
 rec_type_check_enum (rec_type_t type,
-                     char *str,
+                     const char *str,
                      rec_buf_t errors)
 {
   size_t i;
   const char *p, *b;
   char name[100];
 
-  if (!rec_match (str, REC_TYPE_ENUM_VALUE_RE))
+  if (rec_match (str, REC_TYPE_ENUM_VALUE_RE))
     {
-      return false;
-    }
-
-  /* Get the name from STR.  */
-  p = str;
-
-  while (p && rec_blank_p (*p))
-    {
-      p++;
-    }
-
-  b = p;
-  while (p && (rec_letter_p (*p)
-               || rec_letter_p (*p)
-               || rec_digit_p (*p)
-               || (*p == '_')
-               || (*p == '-')))
-    {
-      name[p - b] = *p;
-      p++;
-    }
-  name[p - b] = '\0';
-
-  /* Check for the name in the enum types.  */
-  i = 0;
-  while (type->data.names[i])
-    {
-      if (strcmp (name, type->data.names[i]) == 0)
+      /* Get the name from STR.  */
+      p = str;
+      
+      while (p && rec_blank_p (*p))
         {
-          return true;
+          p++;
         }
       
-      i++;
+      b = p;
+      while (p && (rec_letter_p (*p)
+                   || rec_letter_p (*p)
+                   || rec_digit_p (*p)
+                   || (*p == '_')
+                   || (*p == '-')))
+        {
+          name[p - b] = *p;
+          p++;
+        }
+      name[p - b] = '\0';
+      
+      /* Check for the name in the enum types.  */
+      for (i = 0; i < type->size; i++)
+        if (strcmp (name, type->data.names[i]) == 0)
+          return true;
     }
 
   if (errors)
@@ -1181,22 +1476,23 @@ static const char *
 rec_type_parse_enum (const char *str, rec_type_t type)
 {
   const char *p;
-  size_t i, j;
+  size_t i;
   
   p = str;
 
-  for (i = 0; i < REC_ENUM_MAX_NAMES; i++)
-    {
-      type->data.names[i] = NULL;
-    }
+  type->size = 0;
+  type->data.names = NULL;
 
-  i = 0;
-  while (*p && (i < REC_ENUM_MAX_NAMES))
+  while (*p)
     {
+      /* Allocate space in the list of enum names if needed.  */
+      if ((type->size % REC_ENUM_ALLOC_NAMES) == 0)
+        type->data.names =
+          realloc (type->data.names, ((type->size / REC_ENUM_ALLOC_NAMES) + 1) * (sizeof(char *) * REC_ENUM_ALLOC_NAMES));
+      
       /* Skip blanks.  */
       rec_skip_blanks (&p);
 
-
       if (*p == '(')
         {
           /* Skip the comment.  */
@@ -1220,18 +1516,16 @@ rec_type_parse_enum (const char *str, rec_type_t type)
         {
           /* Parse an enum entry.  */
           if (!rec_parse_regexp (&p,
-                                      "^" REC_TYPE_ENUM_NAME_RE,
-                                      &(type->data.names[i])))
+                                 "^" REC_TYPE_ENUM_NAME_RE,
+                                 &(type->data.names[type->size++])))
             {
               p = NULL;
               break;
             }
-
-          i++;
         }
     }
 
-  if (i == 0)
+  if (type->size == 0)
     {
       /* We require at least one entry in the enum.  In this case it
          is not needed to save memory.  */
@@ -1241,9 +1535,9 @@ rec_type_parse_enum (const char *str, rec_type_t type)
   if (!p)
     {
       /* Free memory.  */
-      for (j = 0; j < i; j++)
+      for (i = 0; i < i; i++)
         {
-          free (type->data.names[j]);
+          free (type->data.names[i]);
         }
     }
 
@@ -1322,6 +1616,44 @@ rec_type_parse_regexp_type (const char *str, rec_type_t type)
   return p;
 }
 
+static const char *
+rec_type_parse_rec (const char *str, rec_type_t type)
+{
+  const char *p = str;
+  
+  /* Get the record name.  */
+  rec_skip_blanks (&p);
+  if (!rec_parse_regexp (&p,
+                         "^" REC_RECORD_TYPE_RE,
+                         &(type->data.recname)))
+    {
+      return NULL;
+    }
+
+  return p;
+}
+
+static bool
+rec_type_parse_range_point (const char **str, int *num)
+{
+  if (rec_match (*str, "^MIN"))
+    {
+      *num = INT_MIN;
+      *str += 3;
+    }
+  else if (rec_match (*str, "^MAX"))
+    {
+      *num = INT_MAX;
+      *str += 3;
+    }
+  else if (!rec_parse_int (str, num))
+    {
+      return false;
+    }
+
+  return true;
+}
+
 static const char *
 rec_type_parse_range (const char *str, rec_type_t type)
 {
@@ -1331,7 +1663,7 @@ rec_type_parse_range (const char *str, rec_type_t type)
 
   rec_skip_blanks (&p);
 
-  if (!rec_parse_int (&p, &(type->data.range[0])))
+  if (!rec_type_parse_range_point (&p, &(type->data.range[0])))
     {
       return NULL;
     }
@@ -1347,7 +1679,7 @@ rec_type_parse_range (const char *str, rec_type_t type)
     }
   else
     {
-      if (!rec_parse_int (&p, &(type->data.range[1])))
+      if (!rec_type_parse_range_point (&p, &(type->data.range[1])))
         {
           return NULL;
         }
