@@ -51,26 +51,26 @@ START_TEST(rec_write_rset_nominal)
 
   record = rec_record_new ();
   fail_if (record == NULL);
-  field = rec_field_new (rec_parse_field_name_str ("foo1"), "value1");
+  field = rec_field_new ("foo1", "value1");
   fail_if (field == NULL);
   fail_if (rec_mset_append (rec_record_mset (record), MSET_FIELD, (void *) field, MSET_ANY) == NULL);
-  field = rec_field_new (rec_parse_field_name_str ("foo2"), "value2");
+  field = rec_field_new ("foo2", "value2");
   fail_if (field == NULL);
   fail_if (rec_mset_append (rec_record_mset (record), MSET_FIELD, (void *) field, MSET_ANY) == NULL);
   fail_if (rec_mset_append (rec_rset_mset (rset), MSET_RECORD, (void *) record, MSET_ANY) == NULL);
 
   record = rec_record_new ();
   fail_if (record == NULL);
-  field = rec_field_new (rec_parse_field_name_str ("bar1"), "value1");
+  field = rec_field_new ("bar1", "value1");
   fail_if (field == NULL);
   fail_if (rec_mset_append (rec_record_mset (record), MSET_FIELD, (void *) field, MSET_ANY) == NULL);
-  field = rec_field_new (rec_parse_field_name_str ("bar2"), "value2");
+  field = rec_field_new ("bar2", "value2");
   fail_if (field == NULL);
   fail_if (rec_mset_append (rec_record_mset (record), MSET_FIELD, (void *) field, MSET_ANY) == NULL);
   fail_if (rec_mset_append (rec_rset_mset (rset), MSET_RECORD, (void *) record, MSET_ANY) == NULL);
   
   writer = rec_writer_new_str (&str, &str_size);
-  fail_if (!rec_write_rset (writer, rset));
+  fail_if (!rec_write_rset (writer, rset, REC_WRITER_NORMAL));
   rec_rset_destroy (rset);
   rec_writer_destroy (writer);
   fail_if (strcmp (str,
