@@ -2,7 +2,7 @@
 #
 # recfix.sh - System tests for recfix.
 #
-# Copyright (C) 2010, 2011, 2012 Jose E. Marchesi.
+# Copyright (C) 2010-2013 Jose E. Marchesi.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 : ${srcdir=.}
 : ${builddir=.}
 : ${crypt_support=yes}
+: ${uuid_support=yes}
 
 . $builddir/config.sh
 . $srcdir/testutils.sh
@@ -33,6 +34,24 @@ test_init "recfix"
 # Create input files.
 #
 
+test_declare_input_file type-rec-valid \
+'%rec: Package
+%type: Maintainer rec Hacker
+
+Name: GNU recutils
+Maintainer: jemarch@gnu.org
+'
+
+test_declare_input_file type-rec-invalid-empty \
+'%rec: Package
+%type: Maintainer rec
+'
+
+test_declare_input_file type-rec-invalid-malformed \
+'%rec: Package
+%type: Maintainer rec foo;invalid
+'
+
 test_declare_input_file type-int-valid \
 '%rec: Types
 %type: Integer int
@@ -664,6 +683,27 @@ Id: 3
 Name: B Field
 '
 
+test_declare_input_file unsorted-int-with-equals \
+'%rec: foo
+%type: Id int
+%sort: Id
+
+Id: 4
+Name: A Field
+
+Id: 2
+Name: C Field
+
+Id: 1
+Name: D Field
+
+Id: 3
+Name: B Field
+
+Id: 2
+Name: Cbis Field
+'
+
 test_declare_input_file unsorted-range \
 '%rec: foo
 %type: Id range 0 10
@@ -682,6 +722,27 @@ Id: 3
 Name: B Field
 '
 
+test_declare_input_file unsorted-range-with-equals \
+'%rec: foo
+%type: Id range 0 10
+%sort: Id
+
+Id: 4
+Name: A Field
+
+Id: 2
+Name: C Field
+
+Id: 1
+Name: D Field
+
+Id: 3
+Name: B Field
+
+Id: 2
+Name: Cbis Field
+'
+
 test_declare_input_file unsorted-real \
 '%rec: foo
 %type: Id real
@@ -700,6 +761,27 @@ Id: 3.2
 Name: B Field
 '
 
+test_declare_input_file unsorted-real-with-equals \
+'%rec: foo
+%type: Id real
+%sort: Id
+
+Id: 4.2
+Name: A Field
+
+Id: 2.2
+Name: C Field
+
+Id: 1.2
+Name: D Field
+
+Id: 3.2
+Name: B Field
+
+Id: 4.2
+Name: Abis Field
+'
+
 test_declare_input_file unsorted-lex \
 '%rec: foo
 %sort: Name
@@ -717,6 +799,26 @@ Id: 3
 Name: B Field
 '
 
+test_declare_input_file unsorted-lex-with-equals \
+'%rec: foo
+%sort: Name
+
+Id: 4
+Name: A Field
+
+Id: 2
+Name: C Field
+
+Id: 1
+Name: D Field
+
+Id: 3
+Name: B Field
+
+Id: 1
+Name: Dbis Field
+'
+
 test_declare_input_file unsorted-bool \
 '%rec: foo
 %type: Bool bool
@@ -762,6 +864,55 @@ Id: 4
 Date: 1 April 1999
 '
 
+test_declare_input_file unsorted-date-with-equals \
+'%rec: foo
+%type: Date date
+%sort: Date
+
+Id: 1
+Date: 24 September 1972
+
+Id: 3
+Date: 23 October 1972
+
+Id: 5
+Date: 26 May 1984
+
+Id: 6
+Date: 23 October 1972
+
+Id: 2
+Date: 23 September 1972
+
+Id: 4
+Date: 1 April 1999
+'
+test_declare_input_file unsorted-multiple \
+'%rec: SortMultiple
+%sort: Class Price
+%type: Price real
+
+Item: one
+Class: B
+Price: 30
+
+Item: two
+Class: A
+Price: 30
+
+Item: three
+Class: C
+Price: 40
+
+Item: four
+Class: A
+Price: 10
+
+Item: five
+Class: B
+Price: 15
+'
+
 test_declare_input_file confidential \
 '%rec: foo
 %confidential: Foo Bar Baz
@@ -875,6 +1026,269 @@ Title: baz
 Title: bar
 '
 
+test_declare_input_file uuid-fields-ok \
+'%rec: Item
+%type: Id uuid
+
+Id: 550e8400-e29b-41d4-a716-446655440000
+Name: Item 1
+
+Id: 550e8401-e29b-41d4-a716-446655440000
+Name: Item 2
+'
+
+test_declare_input_file uuid-fields-invalid \
+'%rec: Item
+%type: Id uuid
+
+Id:
+Name: Item 1
+
+Id: foo
+Name: Item 2
+'
+
+test_declare_input_file type-rec-norset \
+'%rec: foo
+%type: Foo rec bar
+
+Foo: foo
+
+Foo: bar
+'
+
+test_declare_input_file type-rec-nokey \
+'%rec: foo
+%type: Foo rec bar
+
+Foo: foo
+
+Foo: bar
+
+%rec: bar
+
+Bar: 10
+
+Bar: 20
+'
+
+test_declare_input_file type-rec-key-notype \
+'%rec: foo
+%type: Foo rec bar
+
+Foo: foo
+
+Foo: bar
+
+%rec: bar
+%key: Bar
+
+Bar: 10
+
+Bar: 20
+'
+
+test_declare_input_file type-rec-key-type \
+'%rec: foo
+%type: Foo rec bar
+
+Foo: 2
+
+Foo: 3
+
+%rec: bar
+%key: Bar
+%type: Bar int
+
+Bar: 10
+
+Bar: 20
+'
+
+test_declare_input_file type-rec-key-type-invalid \
+'%rec: foo
+%type: Foo rec bar
+
+Foo: xxx
+
+Foo: 3
+
+%rec: bar
+%key: Bar
+%type: Bar int
+
+Bar: 10
+
+Bar: 20
+'
+
+test_declare_input_file constraint-sex-valid \
+'%rec: Task
+%constraint: Status = '\''Closed'\'' => #ClosedBy
+
+Id: 0
+Status: Closed
+ClosedBy: jemarch
+
+Id: 1
+Status: Closed
+ClosedBy: bar
+
+Id: 2
+Status: Open
+'
+
+test_declare_input_file constraint-sex-invalid \
+'%rec: Task
+%constraint: Status &= 'Closed' => #ClosedBy
+
+Id: 0
+Status: Closed
+ClosedBy: jemarch
+
+Id: 1
+Status: Closed
+
+Id: 2
+Status: Open
+'
+
+test_declare_input_file constraint-sex-invalid-empty \
+'%rec: Task
+%constraint:
+
+Id: 0
+Status: Closed
+ClosedBy: jemarch
+
+Id: 1
+Status: Closed
+
+Id: 2
+Status: Open
+'
+
+test_declare_input_file constraint-sex-with-violation \
+'%rec: Task
+%constraint: Status = '\''Closed'\''
++ => #ClosedBy = 0
+
+Id: 0
+Status: Closed
+ClosedBy: jemarch
+
+Id: 1
+Status: Closed
+
+Id: 2
+Status: Open
+'
+
+test_declare_input_file constraint-sex-several-valid \
+'%rec: Task
+%constraint: Status = '\''Closed'\'' => #ClosedBy
+%constraint: 1
+
+Id: 0
+Status: Closed
+ClosedBy: jemarch
+
+Id: 1
+Status: Closed
+ClosedBy: mr foo
+
+Id: 2
+Status: Open
+'
+
+test_declare_input_file constraint-sex-several-invalid \
+'%rec: Task
+%constraint: Status = '\''Closed'\'' => #ClosedBy
+%constraint: Id = 1 => Status != '\''Closed'\''
+
+Id: 0
+Status: Closed
+ClosedBy: jemarch
+
+Id: 1
+Status: Closed
+
+Id: 2
+Status: Open
+'
+
+test_declare_input_file unused-type \
+'%rec: Task
+%typedef: Status_t enum Open Closed
+%key: Id
+
+Id: 1
+'
+
+test_declare_input_file blank-line-after-record \
+'foo: bar
+  
+'
+
+test_declare_input_file blank-line-after-record-invalid \
+'foo:bar
+ x
+'
+
+test_declare_input_file hyphens-in-field-names \
+'foo-bar: baz
+'
+
+test_declare_input_file ranges-min-ok \
+'%rec: foo
+%type: Negative range MIN -1
+
+Negative: -1
+Negative: -2147483648
+'
+
+test_declare_input_file ranges-min-invalid \
+'%rec: foo
+%type: Negative range MIN -1
+
+Negative: -1
+Negative: 0
+'
+
+test_declare_input_file ranges-max-ok \
+'%rec: foo
+%type: Positive range 0 MAX
+
+Positive: 0
+Positive: 2147483647
+'
+
+test_declare_input_file ranges-max-invalid \
+'%rec: foo
+%type: Positive range 0 MAX
+
+Positive: -1
+Positive: 2147483647
+'
+
+test_declare_input_file ranges-minmax-ok \
+'%rec: foo
+%type: Any range MIN MAX
+
+Any: -2147483648
+Any: 0
+Any: 2147483647
+'
+
+test_declare_input_file ranges-minmax-invalid \
+'%rec: foo
+%type None range MAX MIN
+
+None: -2147483648
+None: 0
+None: 2147483647
+'
+
 #
 # Declare tests.
 #
@@ -962,6 +1376,39 @@ test_tool recfix-range-oct-xfail xfail \
           '' \
           ranges-oct-invalid
 
+test_tool recfix-range-min-ok ok \
+          recfix \
+          '' \
+          ranges-min-ok \
+''
+
+test_tool recfix-range-min-invalid xfail \
+          recfix \
+          '' \
+          ranges-min-invalid
+
+test_tool recfix-range-max-ok ok \
+          recfix \
+          '' \
+          ranges-max-ok \
+''
+
+test_tool recfix-range-max-invalid xfail \
+          recfix \
+          '' \
+          ranges-max-invalid
+
+test_tool recfix-range-minmax-ok ok \
+          recfix \
+          '' \
+          ranges-minmax-ok \
+''
+
+test_tool recfix-range-minmax-invalid xfail \
+          recfix \
+          '' \
+          ranges-minmax-invalid
+
 test_tool recfix-ranges-xfail-1 xfail \
           recfix \
           '' \
@@ -1433,6 +1880,59 @@ Id: 4
 Name: A Field
 '
 
+test_tool recfix-sort-multiple ok \
+          recfix \
+          '--sort' \
+          unsorted-multiple \
+'%rec: SortMultiple
+%sort: Class Price
+%type: Price real
+
+Item: four
+Class: A
+Price: 10
+
+Item: two
+Class: A
+Price: 30
+
+Item: five
+Class: B
+Price: 15
+
+Item: one
+Class: B
+Price: 30
+
+Item: three
+Class: C
+Price: 40
+'
+
+test_tool recfix-sort-ints-with-equals ok \
+          recfix \
+          '--sort' \
+          unsorted-int-with-equals \
+'%rec: foo
+%type: Id int
+%sort: Id
+
+Id: 1
+Name: D Field
+
+Id: 2
+Name: C Field
+
+Id: 2
+Name: Cbis Field
+
+Id: 3
+Name: B Field
+
+Id: 4
+Name: A Field
+'
+
 test_tool recfix-sort-ranges ok \
           recfix \
           '--sort' \
@@ -1454,6 +1954,29 @@ Id: 4
 Name: A Field
 '
 
+test_tool recfix-sort-ranges-with-equals ok \
+          recfix \
+          '--sort' \
+          unsorted-range-with-equals \
+'%rec: foo
+%type: Id range 0 10
+%sort: Id
+
+Id: 1
+Name: D Field
+
+Id: 2
+Name: C Field
+
+Id: 2
+Name: Cbis Field
+
+Id: 3
+Name: B Field
+
+Id: 4
+Name: A Field
+'
 
 test_tool recfix-sort-reals ok \
           recfix \
@@ -1476,6 +1999,30 @@ Id: 4.2
 Name: A Field
 '
 
+test_tool recfix-sort-reals-with-equals ok \
+          recfix \
+          '--sort' \
+          unsorted-real-with-equals \
+'%rec: foo
+%type: Id real
+%sort: Id
+
+Id: 1.2
+Name: D Field
+
+Id: 2.2
+Name: C Field
+
+Id: 3.2
+Name: B Field
+
+Id: 4.2
+Name: A Field
+
+Id: 4.2
+Name: Abis Field
+'
+
 test_tool recfix-sort-lex ok \
           recfix \
           '--sort' \
@@ -1496,6 +2043,29 @@ Id: 1
 Name: D Field
 '
 
+test_tool recfix-sort-lex-with-equals ok \
+          recfix \
+          '--sort' \
+          unsorted-lex-with-equals \
+'%rec: foo
+%sort: Name
+
+Id: 4
+Name: A Field
+
+Id: 3
+Name: B Field
+
+Id: 2
+Name: C Field
+
+Id: 1
+Name: D Field
+
+Id: 1
+Name: Dbis Field
+'
+
 test_tool recfix-sort-dates ok \
           recfix \
           '--sort' \
@@ -1520,6 +2090,33 @@ Id: 4
 Date: 1 April 1999
 '
 
+test_tool recfix-sort-dates-with-equals ok \
+          recfix \
+          '--sort' \
+          unsorted-date-with-equals \
+'%rec: foo
+%type: Date date
+%sort: Date
+
+Id: 2
+Date: 23 September 1972
+
+Id: 1
+Date: 24 September 1972
+
+Id: 3
+Date: 23 October 1972
+
+Id: 6
+Date: 23 October 1972
+
+Id: 5
+Date: 26 May 1984
+
+Id: 4
+Date: 1 April 1999
+'
+
 test_tool recfix-sort-booleans ok \
           recfix \
           '--sort' \
@@ -1528,23 +2125,23 @@ test_tool recfix-sort-booleans ok \
 %type: Bool bool
 %sort: Bool
 
+Id: 1
+Bool: 0
+
 Id: 3
 Bool: no
 
 Id: 2
 Bool: false
 
-Id: 1
-Bool: 0
+Id: 5
+Bool: 1
 
 Id: 6
 Bool: true
 
 Id: 4
 Bool: yes
-
-Id: 5
-Bool: 1
 '
 
 test_tool recfix-missing-auto-fields ok \
@@ -1564,6 +2161,119 @@ Id: 2
 Title: bar
 '
 
+if test "$uuid_support" = "yes"; then
+
+test_tool recfix-uuid-ok ok \
+          recfix \
+          '--check' \
+          uuid-fields-ok \
+          ''
+
+test_tool recfix-uuid-invalid xfail \
+          recfix \
+          '--check' \
+          uuid-fields-invalid
+
+fi
+
+test_tool recfix-type-rec-valid ok \
+          recfix \
+          '--check' \
+          type-rec-valid \
+          ''
+
+test_tool recfix-type-rec-invalid-empty xfail \
+          recfix \
+          '--check' \
+          type-rec-invalid-empty
+
+test_tool recfix-type-rec-invalid-malformed xfail \
+          recfix \
+          '--check' \
+          type-rec-invalid-malformed
+
+test_tool recfix-type-rec-norset ok \
+          recfix \
+          '--check' \
+          type-rec-norset \
+          ''
+test_tool recfix-type-rec-nokey ok \
+          recfix \
+          '--check' \
+          type-rec-nokey \
+          ''
+
+test_tool recfix-type-rec-key-notype ok \
+          recfix \
+          '--check' \
+          type-rec-key-notype \
+          ''
+
+test_tool recfix-type-rec-key-type ok \
+          recfix \
+          '--check' \
+          type-rec-key-type \
+          ''
+
+test_tool recfix-type-rec-key-type-invalid xfail \
+          recfix \
+          '--check' \
+          type-rec-key-type-invalid
+
+test_tool recfix-constraint-sex-valid ok \
+          recfix \
+          '--check' \
+          constraint-sex-valid \
+          ''
+
+test_tool recfix-constraint-sex-invalid xfail \
+          recfix \
+          '--check' \
+          constraint-sex-invalid
+
+test_tool recfix-constraint-sex-invalid-empty xfail \
+          recfix \
+          '--check' \
+          constraint-sex-invalid-empty
+
+test_tool recfix-constraint-sex-several-valid ok \
+          recfix \
+          '--check' \
+          constraint-sex-several-valid \
+          ''
+
+test_tool recfix-constraint-sex-several-invalid xfail \
+          recfix \
+          '--check' \
+          constraint-sex-several-invalid
+
+test_tool recfix-constraint-sex-with-violation xfail \
+          recfix \
+          '--check' \
+          constraint-sex-with-violation
+
+test_tool recfix-unused-type ok \
+          recfix \
+          '--check' \
+          unused-type \
+          ''
+
+test_tool recfix-blank-line-after-record ok \
+          recfix \
+         '--check' \
+         blank-line-after-record \
+         ''
+
+test_tool recfix-blank-line-after-record-invalid xfail \
+          recfix \
+         '--check' \
+         blank-line-after-record-invalid
+
+test_tool recfix-hyphens-in-field-names xfail \
+          recfix \
+          '--check' \
+          hyphens-in-field-names
+        
 #
 # Cleanup.
 #
