@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2011, 2012 Jose E. Marchesi */
+/* Copyright (C) 2011-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,8 +50,8 @@ static void rec2csv_generate_csv (rec_rset_t rset, rec_fex_t fex);
  * Global variables
  */
 
-char             *rec2csv_record_type   = NULL;
-rec_field_name_t  rec2csv_sort_by_field = NULL;
+char             *rec2csv_record_type    = NULL;
+rec_fex_t         rec2csv_sort_by_fields = NULL;
 
 /*
  * Command line options management
@@ -96,7 +96,7 @@ Convert rec data into csv data.\n"), stdout);
   fputs (_("\
   -t, --type=TYPE                     record set to convert to csv; if this parameter\n\
                                         is omitted then the default record set is used\n\
-  -S, --sort=FIELD                    sort the output by the specified field.\n"),
+  -S, --sort=FIELDS                   sort the output by the specified fields.\n"),
          stdout);
 
   recutl_print_help_common ();
@@ -130,16 +130,22 @@ rec2csv_parse_args (int argc,
         case SORT_ARG:
         case 'S':
           {
-            if (rec2csv_sort_by_field)
+            if (rec2csv_sort_by_fields)
               {
-                recutl_fatal (_("only one field can be specified as a sorting criteria.\n"));
+                recutl_fatal (_("only one list of fields can be specified as a sorting criteria.\n"));
               }
 
             /* Parse the field name.  */
-            rec2csv_sort_by_field = rec_parse_field_name_str (optarg);
-            if (!rec2csv_sort_by_field)
+
+            if (!rec_fex_check (optarg, REC_FEX_CSV))
               {
-                recutl_fatal (_("invalid field name in -S.\n"));
+                recutl_fatal (_("invalid field name list in -S.\n"));
+              }
+
+            rec2csv_sort_by_fields = rec_fex_new (optarg, REC_FEX_CSV);
+            if (!rec2csv_sort_by_fields)
+              {
+                recutl_fatal (_("internal error creating fex.\n"));
               }
 
             break;
@@ -160,7 +166,7 @@ rec2csv_generate_csv (rec_rset_t rset,
   rec_fex_elem_t fex_elem;
   rec_record_t record;
   rec_field_t field;
-  char *field_name_str;
+  char *field_name;
   char *tmp;
   size_t i;
 
@@ -173,36 +179,32 @@ rec2csv_generate_csv (rec_rset_t rset,
         }
 
       fex_elem = rec_fex_get (fex, i);
-      field_name_str = xstrdup (rec_fex_elem_field_name_str (fex_elem));
+      field_name = xstrdup (rec_fex_elem_field_name (fex_elem));
 
       /* The header is FNAME or FNAME_N where N is the index starting
          at 1.  Note that we shall remove the trailing ':', if any. */
 
-      if (field_name_str[strlen(field_name_str)-1] == ':')
+      if (field_name[strlen(field_name)-1] == ':')
         {
-          field_name_str[strlen(field_name_str)-1] = '\0';
+          field_name[strlen(field_name)-1] = '\0';
         }
 
 
       if (rec_fex_elem_min (fex_elem) != 0)
         {
           if (asprintf (&tmp, "%s_%d",
-                        field_name_str,
+                        field_name,
                         rec_fex_elem_min (fex_elem) + 1) == -1)
-            {
-              recutl_fatal (_("out of memory"));
-            }
+            recutl_out_of_memory ();
         }
       else
         {
-          if (asprintf (&tmp, "%s", field_name_str) == -1)
-            {
-              recutl_fatal (_("out of memory"));
-            }
+          if (asprintf (&tmp, "%s", field_name) == -1)
+            recutl_out_of_memory ();
         }
 
       csv_fwrite (stdout, tmp, strlen(tmp));
-      free (field_name_str);
+      free (field_name);
       free (tmp);
     }
 
@@ -299,7 +301,8 @@ rec2csv_process_data (rec_db_t db)
         {
           /* Process this record set.  */
 
-          rec_rset_sort (rset, rec2csv_sort_by_field);
+          if (!rec_rset_sort (rset, rec2csv_sort_by_fields))
+            recutl_out_of_memory ();
 
           /* Build the fields that will appear in the row. */
           row_fields = rec2csv_determine_fields (rset);
