@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011, 2012 Jose E. Marchesi */
+/* Copyright (C) 2010-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 #include <getopt.h>
 #include <stdlib.h>
 #include <xalloc.h>
-#include <getpass.h>
 #include <gettext.h>
 #define _(str) gettext (str)
 
@@ -128,7 +127,6 @@ Check and fix rec files.\n"),
   /* TRANSLATORS: --help output, recfix global arguments.
      no-wrap */
   fputs (_("\
-  -t, --type=TYPE                     process only records of the given type.\n\
       --no-external                   don't use external descriptors.\n\
       --force                         force the requested operation.\n"),
          stdout);
@@ -304,7 +302,14 @@ recfix_parse_args (int argc,
     {
       if (recutl_interactive ())
         {
-          recfix_password = getpass (_("Password: "));
+          if (recfix_op == RECFIX_OP_ENCRYPT)
+            {
+              recfix_password = recutl_getpass (true);
+            }
+          else
+            {
+              recfix_password = recutl_getpass (false);
+            }
         }
 
       if (!recfix_password || (strlen (recfix_password) == 0))
@@ -388,7 +393,8 @@ recfix_do_sort ()
   for (n_rset = 0; n_rset < rec_db_size (db); n_rset++)
     {
       rset = rec_db_get_rset (db, n_rset);
-      rec_rset_sort (rset, NULL);
+      if (!rec_rset_sort (rset, NULL))
+        recutl_out_of_memory ();
     }
   
   if (!recfix_check_database (db))
@@ -396,6 +402,12 @@ recfix_do_sort ()
       return EXIT_FAILURE;
     }
 
+  if (!recutl_file_is_writable (recfix_file))
+    {
+      recutl_error (_("file %s is not writable.\n"), recfix_file);
+      return EXIT_FAILURE;
+    }
+
   recutl_write_db_to_file (db, recfix_file);
   return EXIT_SUCCESS;
 }
@@ -499,7 +511,8 @@ recfix_do_auto ()
       iter = rec_mset_iterator (rec_rset_mset (rset));
       while (rec_mset_iterator_next (&iter, MSET_RECORD, (const void**) &record, NULL))
         {
-          rec_rset_add_auto_fields (rset, record);
+          if (!rec_rset_add_auto_fields (rset, record))
+            recutl_out_of_memory ();
         }
 
       rec_mset_iterator_free (&iter);
