@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2009, 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -128,7 +128,9 @@ print_info_file (FILE *in,
 
   ret = true;
   parser = rec_parser_new (in, file_name);
-  if (rec_parse_db (parser, &db))
+
+  ret = rec_parse_db (parser, &db);
+  if (ret)
     {
       for (position = 0; position < rec_db_size (db); position++)
         {
@@ -150,6 +152,7 @@ print_info_file (FILE *in,
                 {
                   writer = rec_writer_new (stdout);
                   rec_write_record (writer, descriptor, recinf_write_mode);
+                  rec_write_string (writer, "\n");
                   rec_writer_destroy (writer);
                 }
               else
@@ -273,7 +276,11 @@ main (int argc, char *argv[])
     }
   else
     {
-      print_info_file (stdin, "stdin");
+      if (!print_info_file (stdin, "stdin"))
+        {
+          /* Parse error */
+          exit (EXIT_FAILURE);
+        }
     }
   
   return EXIT_SUCCESS;
