@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011, 2012 Jose E. Marchesi */
+/* Copyright (C) 2010-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
 #include <xalloc.h>
 #include <gettext.h>
 #define _(str) gettext (str)
-#include <getpass.h>
 
 #include <rec.h>
 #include <recutl.h>
@@ -44,8 +43,6 @@ bool recsel_process_data (rec_db_t db);
  * Global variables
  */
 
-bool       recsel_print_values = false;
-bool       recsel_print_row    = false;
 char      *recutl_sex_str      = NULL;
 rec_sex_t  recutl_sex          = NULL;
 char      *recutl_quick_str    = NULL;
@@ -56,11 +53,13 @@ bool       recsel_collapse     = false;
 bool       recsel_count        = false;
 bool       recutl_insensitive  = false;
 bool       recsel_descriptors  = false;
-rec_field_name_t recutl_sort_by_field = NULL;
+rec_fex_t  recutl_sort_by_fields  = NULL;
+rec_fex_t  recsel_group_by_fields = NULL;
 rec_writer_mode_t recsel_write_mode = REC_WRITER_NORMAL;
 char      *recsel_password     = NULL;
 bool       recsel_uniq         = false;
 size_t     recutl_random       = 0;
+char      *recsel_join         = NULL;
 
 /*
  * Command line options management.
@@ -81,7 +80,9 @@ enum
 #if defined REC_CRYPT_SUPPORT
   PASSWORD_ARG,
 #endif
-  UNIQ_ARG
+  UNIQ_ARG,
+  GROUP_BY_ARG,
+  JOIN_ARG
 };
 
 static const struct option GNU_longOptions[] =
@@ -100,6 +101,8 @@ static const struct option GNU_longOptions[] =
     {"password", required_argument, NULL, PASSWORD_ARG},
 #endif
     {"uniq", no_argument, NULL, UNIQ_ARG},
+    {"group-by", required_argument, NULL, GROUP_BY_ARG},
+    {"join", required_argument, NULL, JOIN_ARG},
     {NULL, 0, NULL, 0}
   };
 
@@ -113,7 +116,7 @@ recutl_print_help (void)
   /* TRANSLATORS: --help output, recsel synopsis.
      no-wrap */
   printf (_("\
-Usage: recsel [OPTION]... [-t TYPE] [-n INDEXES | -e RECORD_EXPR | -q EXPR | -m NUM] [-c | (-p|-P) FIELD_EXPR] [FILE]...\n"));
+Usage: recsel [OPTION]... [-t TYPE] [-j FIELD] [-n INDEXES | -e RECORD_EXPR | -q STR | -m NUM] [-c | (-p|-P) FIELD_EXPR] [FILE]...\n"));
 
   /* TRANSLATORS: --help output, recsel arguments.
      no-wrap */
@@ -127,7 +130,8 @@ Select and print rec data.\n"), stdout);
   -d, --include-descriptors           print record descriptors along with the matched\n\
                                         records.\n\
   -C, --collapse                      do not section the result in records with newlines.\n\
-  -S, --sort=FIELD                    sort the output by the specified field.\n\
+  -S, --sort=FIELD,...                sort the output by the specified fields.\n\
+  -G, --group-by=FIELD,...            group records by the specified fields.\n\
   -U, --uniq                          remove duplicated fields in the output records.\n"),
          stdout);
 
@@ -143,6 +147,9 @@ Select and print rec data.\n"), stdout);
 
   puts ("");
   recutl_print_help_record_selection ();
+  fputs (_("\
+  -j, --join=FIELD                    perform an inner join using the specified field.\n"),
+         stdout);
 
   puts ("");
   /* TRANSLATORS: --help output, recsel output options.
@@ -181,7 +188,7 @@ recsel_parse_args (int argc,
                              argv,
                              RECORD_SELECTION_SHORT_ARGS
                              ENCRYPTION_SHORT_ARGS
-                             "S:Cdcp:P:R:U",
+                             "S:Cdcp:P:R:UG:j:",
                              GNU_longOptions,
                              NULL)) != -1)
     {
@@ -223,21 +230,67 @@ recsel_parse_args (int argc,
         case SORT_ARG:
         case 'S':
           {
-            if (recutl_sort_by_field)
+            if (recutl_sort_by_fields)
               {
-                recutl_fatal (_("only one field can be specified as a sorting criteria.\n"));
+                recutl_fatal (_("only one field list can be specified as a sorting criteria.\n"));
               }
 
             /* Parse the field name.  */
-            recutl_sort_by_field = rec_parse_field_name_str (optarg);
-            if (!recutl_sort_by_field)
+
+            if (!rec_fex_check (optarg, REC_FEX_CSV))
               {
-                recutl_fatal (_("invalid field name in -S.\n"));
+                recutl_fatal (_("invalid field names in -S.\n"));
+              }
+
+            recutl_sort_by_fields = rec_fex_new (optarg, REC_FEX_CSV);
+            if (!recutl_sort_by_fields)
+              {
+                recutl_fatal (_("internal error creating fex.\n"));
+              }
+
+            break;
+          }
+        case JOIN_ARG:
+        case 'j':
+          {
+            if (recsel_join)
+              {
+                recutl_fatal (_("only one field can be specified as join criteria.\n"));
+              }
+
+            if (!rec_field_name_p (optarg))
+              {
+                recutl_fatal (_("please specify a correct field name to -j|--join.\n"));
+              }
+
+            recsel_join = xstrdup (optarg);
+            break;
+          }
+        case GROUP_BY_ARG:
+        case 'G':
+          {
+            if (recsel_group_by_fields)
+              {
+                recutl_fatal (_("only one field list can be specified as a grouping criteria.\n"));
+              }
+
+            /* Parse the field name.  */
+            if (!rec_fex_check (optarg, REC_FEX_CSV))
+              {
+                recutl_fatal (_("invalid field names in -G.\n"));
+              }
+
+            recsel_group_by_fields = rec_fex_new (optarg, REC_FEX_CSV);
+            if (!recsel_group_by_fields)
+              {
+                recutl_fatal (_("internal error creating fex.\n"));
               }
 
             break;
           }
         case PRINT_ARG:
+        case PRINT_VALUES_ARG:
+        case PRINT_IN_A_ROW_ARG:
         case 'p':
         case 'P':
         case 'R':
@@ -247,22 +300,21 @@ recsel_parse_args (int argc,
                 recutl_fatal (_("cannot specify -[pPR] and also -c.\n"));
               }
 
-            if (c == 'P')
+            if ((c == 'P') || (c == PRINT_VALUES_ARG))
               {
-                recsel_print_values = true;
+                recsel_write_mode = REC_WRITER_VALUES;
               }
 
-            if (c == 'R')
+            if ((c == 'R') || (c == PRINT_IN_A_ROW_ARG))
               {
-                recsel_print_values = true;
-                recsel_print_row = true;
+                recsel_write_mode = REC_WRITER_VALUES_ROW;
               }
 
             recsel_fex_str = xstrdup (optarg);
 
             if (!rec_fex_check (recsel_fex_str, REC_FEX_SUBSCRIPTS))
               {
-                exit (EXIT_FAILURE);
+                recutl_fatal (_("invalid list of fields in -%c\n"), c);
               }
 
             /* Create the field expresion.  */
@@ -273,6 +325,23 @@ recsel_parse_args (int argc,
                 recutl_fatal (_("internal error creating the field expression.\n"));
               }
 
+            /* Check that all the functions called in the fex exist.
+               Otherwise raise an error.  */
+            
+            {
+              size_t i = 0;
+              for (i = 0; i < rec_fex_size (recsel_fex); i++)
+                {
+                  rec_fex_elem_t elem = rec_fex_get (recsel_fex, i);
+                  const char *fname = rec_fex_elem_function_name (elem);
+
+                  if (fname && !rec_aggregate_std_p (fname))
+                    {
+                      recutl_fatal (_("invalid aggregate function '%s'\n"), fname);
+                    }
+                }
+            }
+
             break;
           }
         case COLLAPSE_ARG:
@@ -300,31 +369,28 @@ recsel_parse_args (int argc,
 
         }
     }
+
+  /* Global checks on the parameters.  */
+
+  if (!recutl_type && recsel_join)
+    {
+      recutl_fatal (_("joins can only be used when a named record set is selected.\n"));
+    }
 }
 
 bool
 recsel_process_data (rec_db_t db)
 {
-  bool ret;
-  int rset_size;
-  rec_rset_t rset;
-  rec_record_t record;
-  int n_rset, written, num_rec;
-  rec_writer_t writer;
-  bool parse_status;
-  bool wrote_descriptor;
-  rec_mset_iterator_t iter;
+  int rset_size = 0;
+  rec_rset_t rset = NULL;
+  rec_writer_t writer = NULL;
 
-  ret = true;
-
-  writer = rec_writer_new (stdout);
 
 #if defined REC_CRYPT_SUPPORT
 
-  /* Set the password in the writer.  If recsel was called
-     interactively and with an empty -s, was not used then prompt the
-     user for it.  Otherwise use the password specified in the command
-     line if any.  */
+  /* If recsel was called interactively and with an empty -s, was not
+     used then prompt the user for it.  Otherwise use the password
+     specified in the command line if any.  */
 
   if (!recsel_password
       && (recutl_type || (rec_db_size (db) == 1))
@@ -342,21 +408,24 @@ recsel_process_data (rec_db_t db)
           rset = rec_db_get_rset (db, 0);
         }
 
-      confidential_fields = rec_rset_confidential (rset);
-      if (rec_fex_size (confidential_fields) > 0)
+      if (rset)
         {
-          recsel_password = getpass (_("Password: "));
+          confidential_fields = rec_rset_confidential (rset);
+          if (rec_fex_size (confidential_fields) > 0)
+            {
+              recsel_password = recutl_getpass (false);
+            }
+          
+          rec_fex_destroy (confidential_fields);
         }
-
-      rec_fex_destroy (confidential_fields);
     }
 
   /* Note that the password must be at least one character long.  */
 
-  if (recsel_password && (strlen (recsel_password) > 0))
+  if (recsel_password && (strlen (recsel_password) == 0))
     {
-      rec_writer_set_password (writer,
-                               recsel_password);
+      free (recsel_password);
+      recsel_password = NULL;
     }
 
 #endif /* REC_CRYPT_SUPPORT */
@@ -364,171 +433,69 @@ recsel_process_data (rec_db_t db)
   /* If the database contains more than one type of records and the
      user did'nt specify the recutl_type then ask the user to clarify
      the request.  */
+
   if (!recutl_type && (rec_db_size (db) > 1))
     {
       recutl_fatal (_("several record types found.  Please use -t to specify one.\n"));
     }
 
-  written = 0;
-  for (n_rset = 0; n_rset < rec_db_size (db); n_rset++)
-    {
-      rset = rec_db_get_rset (db, n_rset);
-      rset_size = rec_rset_num_records (rset);
 
-      wrote_descriptor = false;
+  /* Query the database using the criteria specified by the user in
+     the command line.  */
 
-      /* Don't process empty record sets.  */
-      if (rset_size == 0)
-        {
-          continue;
-        }
+  {
+    int flags = 0;
 
-      /* If the user specified a type, print the record set only if it
-       * is of the given type.  */
-      if (recutl_type
-          && (!rec_rset_type (rset)
-              || (strcmp (recutl_type, rec_rset_type (rset)) != 0)))
-        {
-          continue;
-        }
+    if (recutl_insensitive)
+      {
+        flags = flags | REC_F_ICASE;
+      }
 
-      /* If the user didn't specify a type, print a record set if and
-       * only if:
-       *
-       * - It is the default record set.
-       * - The file contains just one record set.
-       */
+    if (recsel_descriptors)
+      {
+        flags = flags | REC_F_DESCRIPTOR;
+      }
 
-      if (!recutl_type
-          && rec_rset_type (rset)
-          && (rec_db_size (db) > 1))
-        {
-          continue;
-        }
-          
-      /* Process this record set.  */
+    if (recsel_uniq)
+      {
+        flags = flags | REC_F_UNIQ;
+      }
 
-      /* If the user requested to print random records, calculate them
-         now for this record set.  */
-
-      if (recutl_random > 0)
-        {
-          recutl_reset_indexes ();
-          recutl_index_add_random (recutl_random, rec_rset_num_records (rset));
-        }
-
-      rec_rset_sort (rset, recutl_sort_by_field);
-
-      num_rec = -1;
-      iter = rec_mset_iterator (rec_rset_mset (rset));
-      while (rec_mset_iterator_next (&iter, MSET_RECORD, (const void **) &record, NULL))
-        {
-          num_rec++;
-
-          /* Shall we skip this record?  */
-
-          if (recutl_quick_str)
-            {
-              if (!rec_record_contains_value (record,
-                                              recutl_quick_str,
-                                              recutl_insensitive))
-                {
-                  continue;
-                }
-            }
-          else
-            {
-              if (((recutl_num_indexes() != 0) && (!recutl_index_p (num_rec)))
-                  || (recutl_sex_str && !(rec_sex_eval (recutl_sex, record, &parse_status)
-                                          && parse_status)))
-                {
-                  if (recutl_sex_str && (!parse_status))
-                    {
-                      recutl_error (_("evaluating the selection expression.\n"));
-                      return false;
-                    }
-      
-                  continue;
-                }
-            }
-
-          /* Process this record.  */
-          if (recsel_count)
-            {
-              /* We just count this record and continue.  */
-              written++;
-            }
-          else
-            {
-              char *output = NULL;
-
-              /* Remove duplicated fields in the record if requested
-                 by the user.  */
-
-              if (recsel_uniq)
-                {
-                  rec_record_uniq (record);
-                }
-
-              if (recsel_fex_str)
-                {
-                  output = recutl_eval_field_expression (recsel_fex,
-                                                         record,
-                                                         recsel_write_mode,
-                                                         recsel_print_values,
-                                                         recsel_print_row,
-                                                         recsel_password);
-                }
-
-              /* Insert a newline?  */
-              if ((written != 0)
-                  && (!recsel_collapse)
-                  && (!recsel_fex_str || output))
-                {
-                  fprintf (stdout, "\n");
-                }
-
-              /* Write the record descriptor if required.  */
-              if (recsel_descriptors
-                  && !wrote_descriptor
-                  && rec_rset_descriptor (rset))
-                {
-                  rec_write_record (writer, rec_rset_descriptor (rset), recsel_write_mode);
-                  fprintf (stdout, "\n");
-                  wrote_descriptor = true;
-                }
-
-              if (recsel_fex_str)
-                {
-                  /* Print the field expression.  */
-                  if (output)
-                    {
-                      fprintf (stdout, "%s", output);
-                    }
-                }
-              else
-                {
-                  rec_write_record_with_rset (writer,
-                                              rset,
-                                              record,
-                                              recsel_write_mode);
-                }
-
-              written++;
-            }
-        }
-
-      rec_mset_iterator_free (&iter);
-    }
+    rset = rec_db_query (db,
+                         recutl_type,
+                         recsel_join,
+                         recutl_index(),
+                         recutl_sex,
+                         recutl_quick_str,
+                         recutl_random,
+                         recsel_fex,
+                         recsel_password,
+                         recsel_group_by_fields,
+                         recutl_sort_by_fields,
+                         flags);
+    if (!rset)
+      recutl_out_of_memory ();
+  }
 
   if (recsel_count)
     {
-      fprintf (stdout, "%d\n", written);
+      /* Write the number of matching records.  */
+      
+      fprintf (stdout, "%d\n", rec_rset_num_records (rset));
+    }
+  else
+    {
+      /* Write the resulting record set to the standard output.  */
+
+      writer = rec_writer_new (stdout);
+      rec_writer_set_collapse (writer, recsel_collapse);
+      rec_write_rset (writer, rset, recsel_write_mode);
+      rec_writer_destroy (writer);
     }
 
-  rec_writer_destroy (writer);
+  rec_rset_destroy (rset);
 
-  return ret;
+  return true;
 }
 
 int
@@ -549,6 +516,7 @@ main (int argc, char *argv[])
   if (!db)
     {
       res = 1;
+      return res;
     }
 
   /* Process the data.  */
